# SRBunchTrain Machine Procedures
# $Log: not supported by cvs2svn $
# Revision 1.56  2013/01/18 09:26:04  shang
# commented out setting S18 average pv since it no longer exist after S18 was changed to FPGA
#
# Revision 1.55  2011/09/27 02:11:19  sajaev
# Changed S5 bpm sum pvs to S8 in APSBunchTrainConnect because S5 was changed to FPGA.
#
# Revision 1.54  2010/11/02 20:54:23  shang
# added optional of refill in hybrid, injecting to bunch 0 by recalculating total target current.
#
# Revision 1.53  2008/03/11 22:00:53  emery
# Removed global variable for par cleaning switch and added
# a procedure to do a more complicated PAR cleaning turn-on.
#
# Revision 1.52  2004/12/01 17:30:10  emery
# Added PV P:BnchCln:Mux11mbbo (PAR multiplexer that controls bunch cleaning)
# to list of PV to connect with APSBunchTrainConnect.
#
# Revision 1.51  2004/01/23 17:10:41  borland
# Removed a puts to stderr.
#
# Revision 1.50  2003/12/16 03:26:54  emery
# Transfered procedure APSTopupReadbackConnect from
# the application SRBunchtrain.
#
# Revision 1.49  2003/12/16 02:45:16  borland
# When terminating injection, presses the EStop button first to try to
# get reliable termination of the IOC algorithm.
#
# Revision 1.48  2003/10/01 08:58:42  emery
# Added topup PV Mt:SRinjectMultiExcludBsEKBO to allow single bunch
# injection in BTS line.
#
# Revision 1.47  2003/08/05 23:27:20  emery
# Added global statement for apsRFGunKickerHV in procedure APSBunchTrainConnect
#
# Revision 1.46  2003/08/05 19:28:21  emery
# Added Mt:SRinjectDoneBO/apsInjectionDone pair to list of PVs.
#
# Revision 1.45  2003/07/30 02:47:44  emery
# Added Mt:TopUpTime2WarnCalc.VAL to pv linkw list.
#
# Revision 1.44  2002/07/04 16:55:22  borland
# Added options to check PV connectins.
#
# Revision 1.43  2001/11/15 22:29:36  borland
# Added Mt:TopUpTime2WarnInjector.VAL to the list of top-up PVs.
#
# Revision 1.42  2001/11/13 09:38:24  borland
# Added control of whether the linac bunch pattern is reset upon termination.
#
# Revision 1.41  2000/07/21 20:02:15  emery
# Changed the PC gun trigger to the shutter pv: L:LR:shutter4C
#
# Revision 1.40  2000/07/20 21:29:17  emery
# Removed apsBunchTrainGunTrigger global variable
# and changed apsBunchTrainPCTrigger to apsPCGunTrigger.
#
# Revision 1.39  2000/06/27 19:29:05  borland
# Now uses charge disable/enable for RFG kickers.  Also enables HV for the
# kickers, but doesn't normally turn it off.
#
# Revision 1.38  2000/06/21 00:57:57  emery
# replaced L1:${kicker}:KIK:OnBO with L1:RFG:KIK:chargeTimePD.GATE
# since the former controlled several triggers, while
# the latter controls only the kicker discharge which
# is all we want to do.
#
# Revision 1.37  1999/06/01 12:39:48  emery
# Replaced the pv linkw variable for Mt:SRinjectNumBunchesAO
# to apsNumToInject instead of apsTopupBunches after realizing
# that the pv is used more generally.
#
# Revision 1.36  1999/06/01 11:57:13  emery
# *** empty log message ***
#
# Revision 1.35  1999/05/26 21:30:26  borland
# APSTopupConnect now connects to delta current PV.
#
# Revision 1.34  1999/03/19 14:28:33  borland
# Fixed bug in previous version.
#
# Revision 1.33  1999/03/18 22:06:39  soliday
# Modified L1:RFG:KIK:PS:OnBO because of pv name change
#
# Revision 1.32  1999/03/02 04:35:26  emery
# Added L1:RFG:KIK:PS:OnBO pv with tcl variable link apsRFGunKickerTrigger
# to the apsBunchTrainPvList. The rf gun kicker trigger will
# turn off when injection is over. (This is the same behavior as
# the DC gun trigger, which is now obsolete).
#
# Revision 1.31  1998/11/10 16:26:26  emery
# Added procedure APSTopupConnect which adds global variables
# commected to PVs related to topup.
#
# Revision 1.30  1998/07/06 21:51:58  borland
# ADded abort when MPS dump is detected.  Fixed bug with use of
# "caput" where "exec caput" was needed.
#
# Revision 1.29  1998/05/17 18:45:50  borland
# Added code to check S5B:P5 sum signal status.  If it is "Old", then
# IOC-based injection isn't done.
#
# Revision 1.28  1998/04/12 04:14:49  emery
# Resume SROffsetAdjustment controllaw in TerminateInjection
#
# Revision 1.27  1998/04/12 00:29:58  emery
# Use a SetLinacBunches command in TerminateInjection
# to return the number of linac pulses to the number
# at the time of pressing START.
#
# Revision 1.26  1998/04/11 02:39:22  emery
# Removed the setting of bpm sum signal averaging from
# APSBunchTrainConnect procedure. It should be done
# explicitly by the SRBunchTrain application.
#
# Revision 1.25  1998/04/10 23:55:10  emery
# Added procedures GetLinacBunches and SetLinacBunches.
# Also, set the bpm sum averaging to 6 when
# injecting, and reseting it back to 20 when done.
#
# Revision 1.24  1998/03/24 12:14:39  emery
# added BTS chrage to the global list of variables
#
# Revision 1.23  1997/10/15 06:51:52  emery
# Added capability of topping up the BPM bunch train
# using a bpm sum signal (S5B:P5:scdu:ave_sum)
# as a target to reach.
# This makes topping up the BPM bunch train
# independent of the charge in the rest of the ring.
#
# Revision 1.22  1997/06/06 16:55:35  borland
# Added "keep warm" mode so that injector is left running after bpm group
# (or first part of beam) is filled.
#
# Revision 1.21  1997/05/24 18:45:57  borland
# Now turns on/off RF12 to prevent reflected power problems.
#
# Revision 1.20  1997/04/17 14:38:20  borland
# Minor mods to narrow window during which abort function won't work.
#
# Revision 1.19  1997/04/09 21:58:38  borland
# No longer turn off pulsed coil and gun when injection terminate.
#
# Revision 1.18  1997/04/04 14:42:10  borland
# Solved problem with order of use of connect script and 'eval global ...'
# statement in DoBunchTrainInjection.
#
# Revision 1.17  1997/04/04 03:40:41  borland
# Now put a number to Mt:SRinjectMultiEStopBO, since it seems to require
# it.
#
# Revision 1.16  1997/04/04 02:44:51  borland
# Versions that support injection with IOC-based software.  Not completely
# done since IOC sequence program is not finalized.  Has worked, however.
#
# Revision 1.15  1997/03/12 19:19:34  borland
# Now activates and deactivates positron pulsed coil and gun trigger.
#
# Revision 1.14  1996/12/10 17:41:40  borland
# DoInjectionFast returns 1 now if it successfully fills to the desired current.
#
# Revision 1.13  1996/12/04 17:35:50  borland
# Fixed second instance of calculation of par bucket number.
#
# Revision 1.12  1996/12/04 16:57:09  borland
# PAR bucket number is now sent out modulo 36.
#
# Revision 1.11  1996/12/03 18:03:35  emery
# Removed the -1*$bucket%36 calculation for apsBunchPARBucket
# in procedure DoInjection, which was originally implemented
# because the phase shifters are reversed. The ioc will now
# take care of doing this correction.
#
# Revision 1.10  1996/06/24  15:55:00  borland
# Now do multiplet injection here rather than in calling procedure.
# Also added debugging facility.
#
# Revision 1.9  1996/05/23  23:32:57  borland
# Now uses oagwish CA routines.
#
# Revision 1.8  1996/05/19  16:54:51  borland
# Increased delay after septum turn on to allow warmup.
#
# Revision 1.7  1996/05/09  09:58:59  emery
# Added -pendIoTime=5 on all cavput's.
#
# Revision 1.6  1996/03/31  21:47:08  borland
# Removed pointless changes.
#
# Revision 1.4  1996/03/31  04:29:29  borland
# Fixed bug in while statement for cycles loop.
#
# Revision 1.3  1996/03/13  19:27:39  borland
# Added -cycles option for DoBunchTrainInjection to allow repeating the
# injection of the train.
#
# Revision 1.2  1996/01/12  21:15:02  borland
# Added feature to stop injection when a given current limit is reached.
#
# Revision 1.1.1.1  1996/01/03  17:36:42  saunders
# Imported Files
#


proc APSCheckPVConnections {args} {
    set pvList ""
    set pendIOTime 10
    if [APSStrictParseArguments {pvList pendIOTime}] {
        return -code error "APSCheckPVConnections: bad arguments: $args"
    }
    if [catch {exec cavget -list=[join $pvList ,] -pend=10} result] {
        return -code error "APSCheckPVConnections: $result"
    }
    set problemList ""
    foreach pv $pvList value $result {
        if [string compare $value ?]==0 {
            lappend problemList $pv
        }
    }
    return $problemList
}


proc APSBunchTrainConnect {args} {
    set checkPVs 0
    if {[APSStrictParseArguments {checkPVs}]} {
        return -code error "APSBunchTrainConnect: problem with arguments: $args"
    }
    global apsBunchTrainConnected apsBunchTrainPvList apsBunchTrainTclVarList debugMode

    if ![info exists apsBunchTrainConnected] {
        set apsBunchTrainConnected 0
    }

    set debugMode 0

    if {$apsBunchTrainConnected && !$checkPVs} return

    set apsBunchTrainPvList \
      [list Mt:Ddg3chan4.GATE Mt:Ddg3chan0.GATE Mt:Ddg3chan1.GATE \
         It:Bs:BunchSelect_lo Mt:S:BunchSelect_lo PAR:RF1:bucketNumber \
         S35DCCT:currentCC BTS:CM:q S:MPS:readyCC \
         L:LR:shutter4C \
         L1:RFG:KIK:chargeTimePD.GATE L1:RFG:RF:SW2:positionMI \
         Mt:SRinjectMultiEStopBO Mt:SRinjectMultiBO Mt:SRinjectStatMBBI \
         Mt:SRinjectNumBunchesAO Mt:SRinjectDoneBO \
         S28B:P1:Ave:sum \
         S28B:P1:ms.XAVS P:DG1:bDelayAO]

    if $checkPVs {
        if [catch {APSCheckPVConnections -pvList $apsBunchTrainPvList -pendIOTime 10} result] {
            return -code error "APSBunchTrainConnect: $result"
        }
        if [llength $result] {
            return -code error "APSBunchTrainConnect: problems with PVs: $result"
        }
        return
    }

    set apsBunchTrainTclVarList \
      {apsBunchTrainIKTrig apsBunchTrainIS1Trig apsBunchTrainIS2Trig \
         apsBunchTrainBsBucketSel apsBunchTrainSrBucketSel apsBunchPARBucket \
         S35BeamCurrent BTSCharge mpsStatus \
         apsPCGunTrigger \
         apsRFGunKickerTrigger RFGunSelection \
         apsBunchTrainMBStop apsBunchTrainMBInject apsBunchTrainMBStatus \
         apsNumToInject apsInjectionDone \
         apsBPMGroupSum \
         apsBPMGroupSumStatus parBunchCleaningDelay}

    set kicker [exec cavget -list=L1:RFG:RF:SW2:positionMI]
    
    eval global $apsBunchTrainTclVarList
    
    if !$debugMode {
        if {[pv linkw $apsBunchTrainTclVarList $apsBunchTrainPvList]!=0} {
            return -code error "APSBunchTrainConnect: error connecting to PVs $apsBunchTrainPvList"
        }
        if [pv getw $apsBunchTrainTclVarList] {
            return -code error "APSBunchTrainConnect: error reading PVs $apsBunchTrainPvList"
        }
        set RFGunKickerHVPV L1:RG2:KIK:OnBO
        if [string compare $RFGunSelection "RG1"]==0 {
            set RFGunKickerHVPV L1:RG1:KIK:OnBO
        }
        if [pv linkw apsRFGunKickerHV $RFGunKickerHVPV] {
            return -code error "APSBunchTrainConnect: error connecting to PV $RFGunKickerHVPV"
        }
        global apsRFGunKickerHV
        lappend apsBunchTrainTclVarList apsRFGunKickerHV
        lappend apsBunchTrainPvList $RFGunKickerHVPV
    } else {
        if {[pv linkw S35BeamCurrent S35DCCT:currentCC]!=0} {
            return -code error
        }
    }
    
    set apsBunchTrainConnected 1
    return -code ok
}

proc APSTopupConnect {args} {
    set checkPVs 0
    if {[APSStrictParseArguments {checkPVs}]} {
        return -code error "APSTopupConnect:problem with arguments: $args"
    }

    set apsTopupTclVarList \
      {apsTopupEnabled apsTopupTarget apsTopupTolerance apsTopupInterval apsTopupCountdown \
         apsTopupDeltaI apsTopupTime2WarnInjector apsTopupTime2WarnCalc \
         apsBoosterEKexcluded apsTopupInterleaving apsTopupRG2AlphaMagnet
      }
    
    set apsTopupPvList \
      {Mt:TopUpAutoEnableC Mt:TopUpCurrentTargetP Mt:TopUpCurrentToleranceP Mt:TopUpIntervalP Mt:TopUpTime2Inject \
         Mt:TopUpCurrentDeltaIM.VAL Mt:TopUpTime2WarnInjector.VAL Mt:TopUpTime2WarnCalc.VAL \
         Mt:SRinjectMultiExcludBsEKBO.VAL LI:iLeavePermitCC L1:RG2:LFA:CurrentAO
      }

    if $checkPVs {
        if [catch {APSCheckPVConnections -pvList $apsTopupPvList -pendIOTime 10} result] {
            return -code error "APSTopupConnect: $result"
        }
        if [llength $result] {
            return -code error "APSTopupConnect: problem with PVs: $result"
        }
        return
    }

    eval global $apsTopupTclVarList
    if {[pv linkw $apsTopupTclVarList $apsTopupPvList]!=0} {
        return -code error "APSTopupConnect: error connecting to PVs $apsTopupPvList"
    }
    pv getw $$apsTopupTclVarList
    set apsTopupConnected 1
    return -code ok
}

proc APSTopupReadbackConnect {args} {
    set checkPVs 0
    if [APSStrictParseArguments {checkPVs}] {
        return -code error "APSTopupReadbackConnect: bad arguments: $args"
    }

    set apsTopupPvList \
      {Mt:TopUpCurrentTargetP Mt:TopUpCurrentToleranceP Mt:TopUpIntervalP \
         Mt:TopUpWarnTimeP.VAL Mt:TopUpWarnTime2InjectorP.VAL \
         Mt:SRinjectNumBunchesAO.VAL S:IS1:VoltageSetSendAO It:L:bunch_countCC
      }
    # this global list variable is used in SRBunchTrain application
    # in procedures realted to readbacks of top-up related PVs.
    # This variable is not to be confused with the local
    # variable in the procedure APSTopupConnect above.
    global apsTopupTclVarList
    set apsTopupTclVarList \
      {apsTopupTargetReadback apsTopupToleranceReadback apsTopupIntervalReadback \
         apsTopupWarningTimeReadback apsTopupInjectorWarningTimeReadback \
         apsTopupBunchesReadback SIS1SetpointReadback apsLinacBunchesReadback
      }
    
    if $checkPVs {
        if [catch {APSCheckPVConnections -pvList $apsTopupPvList -pendIOTime 10} result] {
            return -code error "APSTopupReadbackConnect: $result"
        }
        if [llength $result] {
            return -code error "APSTopupReadbackConnect: problems with PVs: $result"
        }
        return
    }

    eval global $apsTopupTclVarList
    if {[pv linkw $apsTopupTclVarList $apsTopupPvList]!=0} {
        return -code error "APSTopupReadbackConnect: error connecting to PVs $apsTopupPvList"
    }
    if {[pv getw $apsTopupTclVarList] != 0} {
        return -code error "APSTopupReadbackConnect: $errorCode"
    }
    if {[pv umon $apsTopupTclVarList RefreshTopupReadbacks] != 0} {
        return -code error "APSTopupReadbackConnect: $errorCode"
    }
        set apsTopupConnected 1
    return -code ok
}

proc TerminateInjection {args} {
    set keepWarm 0
    set manageLinacBunches 0
    APSStrictParseArguments {keepWarm manageLinacBunches}

    global KickersOn BunchDone apsBunchTrainTclVarList debugMode apsBunchTrainStopMode errorCode injectionAbort
    global apsLinacBunches
    eval global $apsBunchTrainTclVarList
    set injectionAbort 0
    set apsBunchTrainIKTrig 0
    set apsBunchTrainBsBucketSel 0
    set apsBunchTrainSrBucketSel 0
    set apsBunchPARBucket 0
    set apsBunchTrainIS2Trig 0
    set apsBunchTrainMBStop 1
    if !$keepWarm {
        set apsBunchTrainIS1Trig 0
        set apsPCGunTrigger 0
        set apsRFGunKickerTrigger 0
        set apsRFGunKickerHV 1
    } else {
        # leave the thick septum, gun, and pulsed coil on
        set apsBunchTrainIS1Trig 1
        set apsPCGunTrigger 1
        set apsRFGunKickerTrigger 1
        set apsRFGunKickerHV 1
    }
    if !$debugMode {
        if [pv putw apsBunchTrainMBStop] {
            return -code error "pv putw error in TerminateInjection: $errorCode"
        }
        if [pv putw {apsBunchTrainIKTrig apsBunchTrainBsBucketSel \
                       apsBunchTrainSrBucketSel \
                       apsBunchPARBucket apsBunchTrainIS1Trig apsBunchTrainIS2Trig \
                       apsBunchTrainMBStop}] {
            return -code error "pv putw error in TerminateInjection: $errorCode"
        }
    } else {
        puts stderr "Pretending to terminate injection."
    }
    set KickersOn 0
    set BunchDone 1
    # reset the longer averaging of the sum signal.
    #if [catch {exec cavput -list=S18B:P5:num2ave_ao=20} result] {
    #    APSSetVarAndUpdate bunchTrainStatus "$result"
    #    return 0
    #}
    catch {exec caput S:BPMOffsetAdjustmentsRC.SUSP 0} 
    # reset the original number of linac pulses
    if {$manageLinacBunches && \
          [catch {SetLinacBunches -number $apsLinacBunches } returnValue]} {
        APSSetVarAndUpdate bunchTrainStatus "$returnValue"
        return 0
    }
}

proc PauseInjection {} {
    global KickersOn BunchDone apsBunchTrainTclVarList 
    eval global $apsBunchTrainTclVarList
    set apsBunchTrainIKTrig 0
    set KickersOn 0
    set BunchDone 1
}

proc DoInjection {bucket dwell} {
    global KickersOn BunchDone apsBunchTrainTclVarList debugMode
    eval global $apsBunchTrainTclVarList

    APSBunchTrainConnect

    set apsBunchPARBucket [expr $bucket%36]
    # change bucket selection PVs.

    if !$debugMode {
        set apsBunchTrainBsBucketSel $bucket
        set apsBunchTrainSrBucketSel $bucket
        if [pv putw {apsBunchPARBucket apsBunchTrainBsBucketSel apsBunchTrainSrBucketSel}] {
            global errorCode
            return -code error "pv putw error in DoInjection: $errorCode"
        } 
    } else {
        puts stderr "Pretending to change bucket selection."
    }


    set dwell [expr int($dwell*1e3)]
    if {!$KickersOn} {
        set apsBunchTrainIKTrig 1
        if !$debugMode {
            if [pv putw apsBunchTrainIKTrig] {
                global errorCode
                return -code error "pv putw error in DoInjection: $errorCode"
            }
        } else {
            puts stderr "Pretending to turn on kickers."
        }

        set KickersOn 1
        after 500
    }
    # schedule turn-off of triggers after dwell seconds
    set BunchDone 0
    after $dwell {set BunchDone 1}
}

proc DoBunchTrainInjection {args} {
    global BunchDone InjectionAbort KickersOn apsBunchTrainTclVarList
    global S35BeamCurrent errorCode apsBunchTrainConnected debugMode

    catch {APSBunchTrainConnect}
    eval global $apsBunchTrainTclVarList

    set start 0
    set interval 1
    set number 1
    set dwell 10
    set callback ""
    set stopAt 0
    set stopAtSum 0
    set useSum DCCT
    set cycles 1
    set multiplet 1
    set keepWarm 0
    set manageLinacBunches 0
#    puts stderr "DoBunchTrainInjection: $args"
    APSParseArguments {start interval number dwell callback fast stopAt stopAtSum \
                         useSum cycles multiplet keepWarm manageLinacBunches}

    if {$start<0} { set start 0}
    set start [expr $start%1295]
    if {$interval<0} { set interval 0}
    set interval [expr $interval%1296]
    if {$dwell<0} { set dwell 1 }
    if {$stopAt<0} { set stopAt 0 }
    if {$stopAtSum<0} { set stopAtSum 0 }
    if {$cycles<=0} {set cycles 1}
    if {$multiplet<=0} {set multiplet 1}

    set bucket $start
    set BunchDone 0
    set KickersOn 0
    set loopTime 200

    # turn on septum discharge and gun/pulsed-coil and allow warmup time
    if !$debugMode {
        set apsBunchTrainIS1Trig 1
        set apsBunchTrainIS2Trig 1
        set apsPCGunTrigger 1
        set apsRFGunKickerTrigger 1
        set apsRFGunKickerHV 1
        if [pv putw {apsBunchTrainIS1Trig apsBunchTrainIS2Trig \
                       apsRFGunKickerTrigger apsRFGunKickerHV \
                       apsPCGunTrigger}] {
            global errorCode
            return -code error "pv putw error in DoInjection: $errorCode"
        }
    } else {
        puts stderr "Pretending to turn on septa."
    }
    after 3500

    set number0 $number
    set bucket0 $bucket
    set multiplet0 $multiplet
    set currentAchieved 0
    while {!$InjectionAbort && $cycles>0} {
        set multiplet $multiplet0
        while {$multiplet>0} {
            set number $number0
            set bucket [expr $bucket0+($multiplet0-$multiplet)]
            while {[expr $number > 0] } {
                if {$callback!=""} { $callback $bucket }
                DoInjection $bucket $dwell
                while {!$BunchDone} {
                    if [pv getw mpsStatus] {
                        return -code error
                    }
                    set mpsStatus $mpsStatus
                    if !$mpsStatus {
                        set InjectionAbort 1
                        if {$callback!=""} { $callback "aborted due to MPS trip"}
                        TerminateInjection -keepWarm $keepWarm -manageLinacBunches $manageLinacBunches
                        return 0
                    }
                    if {$useSum!="Sum"} {
                        if $stopAt {
                            if [pv getw S35BeamCurrent] {
                                return -code error
                            } else {
                                set S35BeamCurrent $S35BeamCurrent
                                set current [expr abs($S35BeamCurrent)]
                                if {$stopAt<=$current} {
                                    set currentAchieved 1
                                    set InjectionAbort 1
                                }
                            }
                        } else {
                            if [pv getw apsBPMGroupSum] {
                                return -code error
                            } else {
                                set apsBPMGroupSum $apsBPMGroupSum
                                if {$stopAtSum<=$apsBPMGroupSum} {
                                    set currentAchieved 1
                                    set InjectionAbort 1
                                }
                            }
                        }
                    }
                    if {$InjectionAbort} {
                        TerminateInjection -keepWarm $keepWarm -manageLinacBunches $manageLinacBunches
                        return 0
                    }
                    update
                    after $loopTime
                }
                if {$InjectionAbort} {
                    TerminateInjection -keepWarm $keepWarm -manageLinacBunches $manageLinacBunches
                    return 0
                }
                set bucket [expr ($bucket+$interval)%1296]
                set number [expr $number - 1]
            }
            incr multiplet -1
        }
        incr cycles -1
    }
    set returnValue 1
    if {$InjectionAbort} {
        set returnValue 0
    }
    TerminateInjection -keepWarm $keepWarm -manageLinacBunches $manageLinacBunches
    return $returnValue
}

proc GetLinacBunches {} {
    if [catch {exec cavget -list=It:L:bunch_countCC \
             } result] {
        return -code error $result
    }
    return $result
}

proc SetLinacBunches {args} {
    set number 0
    APSParseArguments {number}
    if {$number <0 } {set number 0}
    if {$number > 15} {set number 15}
    set endBunch [expr $number * 2 - 2]
    set endBunchp2 [expr $number * 2 ]
    if [catch {exec cavput -list=It:L:B \
                 -range=begin=0,end=$endBunch,interval=2,format=%02ld \
                 -list=_enb_bo=On \
             } result] {
        return -code error $result
    }
    if [catch {exec cavput -list=It:L:B \
                 -range=begin=$endBunchp2,end=28,interval=2,format=%02ld \
                 -list=_enb_bo=Off \
             } result] {
        return -code error $result
    }
}

set InjectionAbort 0
proc AbortInjection {} {
    global InjectionAbort
    set InjectionAbort 1
}


proc APSPARCleaning {args} {
    set state 0
    APSParseArguments {state}
    if $state {
        if [catch {exec APScavput -list=P:Ds345:AmplitudeRmsC=0.23 \
                 } result] {
            return -code error "APSPARCleaning: $result"
        }
    } else {
        if [catch {exec APScavput -list=P:Ds345:AmplitudeRmsC=0.04 \
                 } result] {
            return -code error "APSPARCleaning: $result"
        }
    }
}
