#
# $Log: not supported by cvs2svn $
# Revision 1.7  2001/11/09 07:50:49  borland
# Fixed bug (was collecting tune data when not asked to do so).
#
# Revision 1.6  1999/07/28 00:08:47  borland
# Added ability to vary a PV during measurements.
#
# Revision 1.5  1998/10/20 17:38:16  borland
# Fixed some bugs that weren't bugs 1 month ago.
#
# Revision 1.4  1998/09/07 21:00:30  borland
# Now processes scraper position data in addition to current and current
# per bunch.
#
# Revision 1.3  1998/08/10 16:29:49  borland
# Fixed a bug in the name of a procedure call.
#
# Revision 1.2  1998/08/10 14:52:25  borland
# Now accesses the input files from a central location in oagData.
#
# Revision 1.1  1998/08/10 14:42:38  borland
# First version of SRLifetime.tcl.  Added it to Makefile.Unix.
#

proc APSSRTakeLifetimeDataSeries {args} {
    set buttonWidget ""
    set minCurrentChange 0.2
    set maxCurrentChange 1
    set maxTime 300
    set minTime 10
    set timeInterval 0.5
    set takeTunes 1
    set takeOrbit 1
    set takeBunchPattern 1
    set takeSRVSA 1
    set rootname ""
    set comment ""
    set sets 1
    set timeBetweenSets 60
    set abortVariable ""
    set statusCallback APSNoOp
    set plot 0
    set PVToVary ""
    set PVToVaryStart 0
    set PVToVaryEnd 0
    set postChangePause 0
    APSStrictParseArguments {buttonWidget minCurrentChange maxCurrentChange \
                               maxTime minTime timeInterval \
                               takeTunes takeOrbit takeBunchPattern \
                               rootname comment sets takeSRVSA \
                               timeBetweenSets abortVariable statusCallback plot \
                               PVToVary PVToVaryStart PVToVaryEnd postChangePause}

    if {$maxTime<$minTime || $timeInterval<0 || ![string length $rootname] || \
        $minCurrentChange>$maxCurrentChange || \
        $sets<1 || $timeBetweenSets<0} {
        return -code error "APSSRTakeLifetimeDataSeries: bad argument values"
    }
    
    if [string length $buttonWidget] {
        APSDisableButton $buttonWidget
    }
    if [string length $abortVariable] {
        global $abortVariable
        set $abortVariable 0
    }

    set fileIndex 0
    set existingFileList [lsort [glob -nocomplain $rootname.\[0-9\]\[0-9\]\[0-9\].monData]]
    if [llength $existingFileList] {
        # get the last generation number 
        scan [lindex $existingFileList end] $rootname.%ld.monData fileIndex
        incr fileIndex
    }

    if [string length $PVToVary] {
        set PVValue $PVToVaryStart
        set PVDelta 0
        if [string compare $PVToVary RfVoltage]==0 {
            if [catch {exec cavget -list=SRF:S -list=36,37,38,40 -list=:cavTotalGV:SetptAO} resultList] {
                return -code error "$resultList"
            }
            set totalVoltage 0
            foreach result $resultList {
                set totalVoltage [expr $totalVoltage+$result]
            }
            set PVOrigValue [expr $totalVoltage/4.0]
        } else {
            if [catch {exec cavget -list=$PVToVary} PVOrigValue] {
                if [string length $buttonWidget] {
                    APSEnableButton $buttonWidget
                }
                return -code error "APSSRTakeLifetimeDataSeries: $PVOrigValue"
            }
        }
        if $sets>1 {
            set PVDelta [expr ($PVToVaryEnd-$PVToVaryStart)/($sets-1.0)]
        } 
    }
    for {set dataset 1} {$dataset<=$sets} {incr dataset; incr fileIndex} {
        if [string length $PVToVary] {
            if [string compare $PVToVary RfVoltage]==0 {
                exec cavput -list=SRF:S -list=36,37,38,40 -list=:cavTotalGV:SetptAO=[expr $PVValue/4.0]
            } else {
                if [catch {exec cavput -list=$PVToVary=$PVValue} result] {
                    if [string length $buttonWidget] {
                        APSEnableButton $buttonWidget
                    }
                    catch {exec cavput -list=$PVToVary=$PVOrigValue}
                    return -code error "APSSRTakeLifetimeDataSeries: $result"
                }
            }
            set PVValueNow $PVValue
            set PVValue [expr $PVValue+$PVDelta]
            after [expr int($postChangePause*1000)]
        }
        set basename [format $rootname.%03ld $fileIndex]
        eval $statusCallback {"Taking set $dataset in files $basename.*"}
        if [catch {APSSRTakeLifetimeData -rootname $basename \
                     -minTime $minTime -maxTime $maxTime \
                     -minCurrentChange $minCurrentChange \
                     -maxCurrentChange $maxCurrentChange \
                     -takeTunes $takeTunes -takeOrbit $takeOrbit \
                     -takeBunchPattern $takeBunchPattern \
                     -takeSRVSA $takeSRVSA \
                     -timeInterval $timeInterval \
                     -abortVariable $abortVariable \
                     -PVToVary $PVToVary \
                     -statusCallback $statusCallback \
                     -plot $plot -comment $comment} result] {
            if [string length $buttonWidget] {
                APSEnableButton $buttonWidget
            }
            catch {exec cavput -list=$PVToVary=$PVOrigValue}
            return -code error "APSSRTakeLifetimeDataSeries: $result"
        }
        eval $statusCallback {"Done with set $dataset"}
        if [string length $PVToVary] {
            if [catch {exec sddsprocess $basename.monData -nowarning \
                         -define=column,$PVToVary,$PVValueNow} result] {
                eval $statusCallback {"Warning: problem setting $PVToVary in output file $basename.monData: $result."}
            } else {
                catch {file delete -force $basename.monData~}
            }
            if [catch {exec sddsprocess $basename.ltfit -nowarning \
                         -define=param,$PVToVary,$PVValueNow} result] {
                eval $statusCallback {"Warning: problem setting $PVToVary in output file $basename.ltfit: $result."}
            } else {
                catch {file delete -force $basename.ltfit~}
            }
        }
        if {$timeBetweenSets && $dataset!=$sets} {
            eval $statusCallback {"Waiting $timeBetweenSets seconds"}
            APSWaitWithUpdate -waitSeconds $timeBetweenSets -updateInterval 1 \
              -abortVariable $abortVariable
        }
        if {[string length $abortVariable] && [set $abortVariable]} {
            break
        }
    }
    if {[string length $abortVariable] && [set $abortVariable]} {
        eval $statusCallback {"Measurement aborted."}
    } else {
        eval $statusCallback {"Measurement done."}
    }
    if [string length $buttonWidget] {
        APSEnableButton $buttonWidget
    }
    if [string length $PVToVary] {
        if [string compare $PVToVary RfVoltage]==0 {
            exec cavput -list=SRF:S -list=36,37,38,40 -list=:cavTotalGV:SetptAO=$PVOrigValue
        } else {
            catch {exec cavput -list=$PVToVary=$PVOrigValue -ramp=steps=10,pause=1}
        }
    }
}

proc APSSRTakeLifetimeData {args} {
    set rootname ""
    set minCurrentChange 0.2
    set maxCurrentChange 1
    set maxTime 300
    set minTime 10
    set timeInterval 0.5
    set takeTunes 1
    set takeOrbit 1
    set takeBunchPattern 1
    set takeSRVSA 1
    set abortVariable ""
    set plot 0
    set statusCallback APSNoOp
    set PVToVary ""
    set comment ""
    APSStrictParseArguments {rootname minCurrentChange maxCurrentChange maxTime minTime \
                               timeInterval takeTunes takeOrbit takeBunchPattern \
                               abortVariable statusCallback plot comment takeSRVSA PVToVary}
    set inputDir \
      /home/helios/oagData/sr/sddsmonitorFiles/lifetimeDataCollection

    if [string length $abortVariable] {
        global $abortVariable 
    } else {
        set abortVariable abortDummy
        set abortDummy 0
    }

    if {$maxTime<$minTime || $timeInterval<0 || ![string length $rootname] || \
        $minCurrentChange>$maxCurrentChange} {
        return -code error "APSSRTakeLifetimeData: bad argument values"
    }

    if {[pv linkw S35DCCT S35DCCT:currentCC]!=0} {
        return -code error "APSSRTakeLifetimeData: Unable to connect to DCCT"
    }

    set timeStart [clock seconds]
    set timeEnd [expr $timeStart+$maxTime]
    set timeMustReach [expr $timeStart+$minTime]
    global S35DCCT
    if [pv getw S35DCCT] {
        return -code error "APSSRTakeLifetimeData: Unable to read DCCT"
    }
    set currentStart [set S35DCCT $S35DCCT]
    set currentEnd [expr $currentStart - $maxCurrentChange]
    if {$currentEnd<0} {
        set currentEnd 0
    }
    set currentMustReach [expr $currentStart - $minCurrentChange]
    if {$currentMustReach<0} {
        set currentMustReach 0
    }
    set smPID [exec sddsmonitor $inputDir/lifetime.mon \
                 "-comment=Comment,$comment" \
                 $rootname.monData -interval=$timeInterval,s \
                 -time=$maxTime,s -precision=single -enforceTimeLimit &]
    if $takeTunes {
        if [catch {APSMpMeasureTunes -ring SR -averagingSeconds -1} result] {
            return -code error "APSSRTakeLifetimeData: Unable to trigger tune measurement: $result"
        }
    }
    if $takeOrbit {
        if [catch {exec sddsmonitor -step=1 $inputDir/orbit.mon \
                     "-comment=Comment,$comment" \
                     $rootname.orbit -precision=single} result] {
            return -code error "APSSRTakeLifetimeData: Unable to read orbit: $result"
        }
    }
    if $takeBunchPattern {
        if ![catch {APSSRReadBunchCM -output $rootname.bpat} result] {
            if $plot {
                exec sddsplot -column=Bucket,BunchCurrent $rootname.bpat &
            }
        }
    }
    while {![set $abortVariable]} {
        set timeNow [clock seconds]
        if [pv getw S35DCCT] {
            catch {exec kill -9 $smPID}
            return -code error "APSSRTakeLifetimeData: Unable to read DCCT"
        }
        set current [set S35DCCT $S35DCCT]
        eval $statusCallback {"[format %.2f $current] ([format %.2f $currentMustReach]) mA after [format %.0f [expr $timeNow-$timeStart]]s"}
        if {$current<$currentEnd} {
            eval $statusCallback {"Limiting current loss seen."}
            break
        }
        if {$timeNow>$timeEnd} {
            eval $statusCallback {"Limiting elapsed time has passed."}
            break
        }
        if {$timeNow>$timeMustReach && $current<$currentMustReach} {
            eval $statusCallback {"Desired current loss reached."}
            break
        }
        after 1000
    }
    catch {exec kill -9 $smPID}
    if $takeTunes {
        if [catch {APSMpMeasureTunes -ring SR -averagingSeconds -1 \
                     -useExistingData 1 -plotData $plot \
                     -outputFile $rootname.tunes} result] {
            return -code error "APSSRTakeLifetimeData: Unable to get tune data: $result"
        }
    }
    if $takeSRVSA {
        if [catch {exec hpVecTrace -hpvecsr a $rootname.vsa} result] {
            return -code error "APSSRTakeLifetimeData: Unable to get VSA data: $result"
        }
    }
    if [catch {APSSRProcessLifetimeData -input $rootname.monData \
                 -output $rootname.ltfit} result] {
        return -code error "APSSRTakeLifetimeData: $result"
    }
    if $plot {
        exec sddsplot -column=TimeElapsed,lnS35DCCT $rootname.ltfit \
            -graph=dot -topline=@LifetimeLabel -title=@Comment \
            -column=TimeElapsed,lnS35DCCTFit -graph=line,type=1 $rootname.ltfit &
    }
}

proc APSSRProcessLifetimeData {args} {
    set input ""
    set output ""
    set PVToVary "" 
    APSStrictParseArguments {input output PVToVary}

    if ![file exists $input] {
        return -code error "APSSRProcessLifetimeData: not found: $input"
    }
    set tmpFile /tmp/[APSTmpString]
    if ![string length $PVToVary] {
        if [catch {exec sddsprocess $input $tmpFile \
                       -process=Time,first,Time0 \
                       "-define=column,TimeElapsed,Time Time0 -,units=s" \
                       "-define=column,lnS35DCCT,S35DCCT ln" \
                       "-redefine=column,SRBucketsFilled,SRBucketsFilled 0 > ? SRBucketsFilled : 1 $ " \
                       "-define=column,BunchCurrent,S35DCCT SRBucketsFilled /,units=mA" \
                       -process=S35DCCT,ave,S35DCCTMean \
                       -process=BunchCurrent,ave,BunchCurrentMean \
                       -process=*SC*Setpoint,ave,%sMean
            exec sddsconvert $tmpFile -pipe=out -retain=column,TimeElapsed,lnS35DCCT \
                       | sddspfit -pipe -generate \
                       -terms=2 -columns=TimeElapsed,lnS35DCCT \
                       | sddsxref -pipe $tmpFile -leave=* \
                       -transfer=param,S35DCCTMean,Comment,BunchCurrentMean,*SC*SetpointMean \
                       | sddsprocess -pipe=in $output \
                       "-define=parameter,Rate,Slope -3600 *,units=1/h" \
                       "-define=parameter,RateSigma,SlopeSigma 3600 *,units=1/h" \
                       "-define=parameter,Lifetime,Slope rec chs 3600 /,units=h" \
                       "-define=parameter,LifetimeSigma,SlopeSigma Slope sqr / 3600 /,units=h" \
                       "-define=parameter,LifetimeTimesCurrent,Lifetime S35DCCTMean *,units=mA-h" \
                       "-define=parameter,LifetimeTimesCurrentSigma,LifetimeSigma S35DCCTMean *,units=mA-h" \
                       "-print=parameter,LifetimeLabel,%.3f +/- %.3f h   %.3f +/- %.3f mA-h,Lifetime,LifetimeSigma,LifetimeTimesCurrent,LifetimeTimesCurrentSigma"} result] {
            catch {exec rm $tmpFile}
            return -code error "APSSRProcessLifetimeData: Lifetime data processing error: $result"
        }
    } else {
        if [catch {exec sddsprocess $input $tmpFile \
                       -process=Time,first,Time0 \
                       -process=$PVToVary,first,$PVToVary \
                       "-define=column,TimeElapsed,Time Time0 -,units=s" \
                       "-define=column,lnS35DCCT,S35DCCT ln" \
                       "-redefine=column,SRBucketsFilled,SRBucketsFilled 0 > ? SRBucketsFilled : 1 $ " \
                       "-define=column,BunchCurrent,S35DCCT SRBucketsFilled /,units=mA" \
                       -process=S35DCCT,ave,S35DCCTMean \
                       -process=BunchCurrent,ave,BunchCurrentMean \
                       -process=*SC*Setpoint,ave,%sMean
            exec sddsconvert $tmpFile -pipe=out -retain=column,TimeElapsed,lnS35DCCT \
                       | sddspfit -pipe -generate \
                       -terms=2 -columns=TimeElapsed,lnS35DCCT \
                       | sddsxref -pipe $tmpFile -leave=* \
                       -transfer=param,S35DCCTMean,Comment,BunchCurrentMean,*SC*SetpointMean,$PVToVary \
                       | sddsprocess -pipe=in $output \
                       "-define=parameter,Rate,Slope -3600 *,units=1/h" \
                       "-define=parameter,RateSigma,SlopeSigma 3600 *,units=1/h" \
                       "-define=parameter,Lifetime,Slope rec chs 3600 /,units=h" \
                       "-define=parameter,LifetimeSigma,SlopeSigma Slope sqr / 3600 /,units=h" \
                       "-define=parameter,LifetimeTimesCurrent,Lifetime S35DCCTMean *,units=mA-h" \
                       "-define=parameter,LifetimeTimesCurrentSigma,LifetimeSigma S35DCCTMean *,units=mA-h" \
                       "-print=parameter,LifetimeLabel,%.3f +/- %.3f h   %.3f +/- %.3f mA-h,Lifetime,LifetimeSigma,LifetimeTimesCurrent,LifetimeTimesCurrentSigma"} result] {
            catch {exec rm $tmpFile}
            return -code error "APSSRProcessLifetimeData: Lifetime data processing error: $result"
        }
    }
}
