# $Log: not supported by cvs2svn $
# Revision 1.3  2009/12/14 22:40:26  soliday
# Removed the standardize command completely
#
# Revision 1.2  2009/12/14 22:37:49  soliday
# Added a check to see if the standardize file is empty.
#
# Revision 1.1  2002/09/12 18:55:21  soliday
# First version.
#
#
#

proc APSMpLTS_TurnOnPSInfo {} {
    return "This procedure allows turning on Test Stand power supplies in groups.  You may choose from among L1:PC1 LTS:H LTS:V LTS:Q LTS:BM1."
}

proc APSMpLTS_TurnOnPS {args} {
    set arrayName APSMpLTSControlPS
    set configured 0
    set turnOnList ""
    set turnOnListDetailed ""
    APSStrictParseArguments {arrayName configured turnOnList turnOnListDetailed}

    global status mpInterface execMode $arrayName
    if {![info exists $arrayName] || !$configured} {
        APSMpGenericPSArraySetup -arrayName $arrayName \
	    -turnOnList $turnOnList \
	    -turnOnListDetailed $turnOnListDetailed \
	    -system lts
    }

    APSMpStep init -promptDialog \
      "APSMpGenericPSDialog -turnOn 1 -arrayName $arrayName -system LTS -frame "

    set turnOnList [APSMpGenericPSArrayQuery -arrayName $arrayName -operation TurnOn]

    foreach sector "L1:PC1 LTS:H LTS:V LTS:Q LTS:BM1" {
	if {[lsearch $turnOnList $sector] != -1} {
	    if [catch {APSDevSend -releaseLinks 1 \
			   -group PSOps \
			   -operation QueryOn \
			   -deviceList LTS:RAW} result] {
		APSMpReturn error "APSMpLTS_TurnOnPS: $result"
	    }
	    if {!$result} {
		if [catch {APSDevSend -releaseLinks 1 \
			       -group PSOps \
			       -operation TurnOn \
			       -deviceList LTS:RAW} result] {
		    APSMpReturn error "APSMpLTS_TurnOnPS: $result"
		}
                APSWaitWithUpdate -waitSeconds 3
	    }
	    break
	}
    }
    set turnOnList [APSMpConvertDeviceNames \
			-operation TurnOn \
			-deviceList $turnOnList \
			-system lts]
    set turnOnList [APSMpConvertDetailedDeviceNames \
			-operation TurnOn \
			-deviceList $turnOnList \
			-arrayName $arrayName]

    if [catch {APSDevSend -releaseLinks 1 \
		   -timeout 10 \
		   -group PSOps \
		   -operation QueryOn \
		   -deviceList $turnOnList} result] {
	APSMpReturn error "APSMpLTS_TurnOnPS: $result"
    }
    
    set tol ""
    foreach item $result device $turnOnList {
	if {!$item} {
	    lappend tol $device
	}
    }
    if {[llength $tol]} {
	if [catch {APSDevSend -releaseLinks 1 \
		       -timeout 10 \
		       -group PSOps \
		       -operation TurnOn \
		       -deviceList $tol} result] {
	    APSMpReturn error "APSMpLTS_TurnOnPS: $result"
	}
    }
    APSMpReturn ok
}

proc APSMpLTS_TurnOffPSInfo {} {
    return "This procedure allows turning off Test Stand power supplies in groups.  You may choose from among L1:PC1 LTS:H LTS:V LTS:Q LTS:BM1."
}

proc APSMpLTS_TurnOffPS {args} {
    set arrayName APSMpLTSControlPS
    set configured 0
    set turnOffList ""
    set turnOffListDetailed ""
    APSStrictParseArguments {arrayName configured turnOffList turnOffListDetailed}
    global status mpInterface execMode $arrayName
    if {![info exists $arrayName] || !$configured} {
        APSMpGenericPSArraySetup -arrayName $arrayName \
	    -turnOffList $turnOffList \
	    -turnOffListDetailed $turnOffListDetailed \
	    -system lts
    }
    if {[llength $turnOffList] == 0} {
	APSMpStep init -promptDialog \
	    "APSMpGenericPSDialog -turnOff 1 -arrayName $arrayName -system LTS -frame "
	set turnOffList [APSMpGenericPSArrayQuery -arrayName $arrayName -operation TurnOff]
    }

    set foundall 1
    foreach sector "L1:PC1 LTS:H LTS:V LTS:Q LTS:BM1" {
	if {[lsearch $turnOffList $sector] == -1} {
	    set foundall 0
	    break
	}
	set detailedDeviceList [APSMpConvertDeviceNames \
				    -operation TurnOff \
				    -deviceList $sector \
				    -system lts]
	foreach detailedDevice $detailedDeviceList {
	    if {[set ${arrayName}(TurnOffDetailed,$detailedDevice)] == 0} {
		set foundall 0
		break
	    }
	}
    }
    if {$foundall} {
	if [catch {APSDevSend -releaseLinks 1 \
		       -group PSOps \
		       -operation TurnOff \
		       -deviceList LTS:RAW} result] {
	    APSMpReturn error "APSMpLTS_TurnOffPS: $result"
	}
    }

    APSMpReturn ok [APSMpArrayDrivenOperationPS \
			-arrayName $arrayName \
			-operation TurnOff \
			-system lts]
}

proc APSMpLTS_ConditionPowerSuppliesInfo {} {
    return "This procedure allows conditioning up Test Stand power supplies in groups.  You may choose from among L1:PC1 LTS:H LTS:V LTS:Q LTS:BM1.  In addition, you may choose an SCR file from which to restore power supply setpoints.  The conditioning parameters are also configured."
}

proc APSMpLTS_ConditionPowerSupplies {args} {
    set SCRFile ""
    set configured 0
    set conditionList ""
    set conditionListDetailed ""
    set turnOnList ""
    set turnOnListDetailed ""
    set conditionList ""
    set cycles 2

    APSStrictParseArguments {conditionList conditionListDetailed SCRFile configured cycles}
    
    if {[llength $conditionList] == 0} {
	set conditionList [list L1:PC1 LTS:H LTS:V LTS:Q LTS:BM1]
	set autoAdd 1
    }

    global status mpInterface execMode APSMpLTSControlPS
    set turnOnList $conditionList
    set turnOnListDetailed $conditionListDetailed
    set arrayName APSMpLTSControlPS
    if {![info exists $arrayName] || !$configured} {
	APSMpGenericPSArraySetup -arrayName $arrayName \
	    -turnOnList $turnOnList \
	    -turnOnListDetailed $turnOnListDetailed \
	    -conditionList $conditionList \
	    -conditionListDetailed $conditionListDetailed \
	    -SCRFile $SCRFile \
	    -cycles $cycles \
	    -system lts
    }

    APSMpStep init -promptDialog \
      "APSMpGenericPSDialog -turnOn 0 -condition 1 -chooseSCR 1 -arrayName $arrayName -system LTS -frame "
    set SCRFile $APSMpLTSControlPS(Filename)

    set conditionList [APSMpGenericPSArrayQuery \
			   -arrayName $arrayName \
			   -operation StartConditioning]
    set cld [APSMpConvertDeviceNames \
		 -operation StartConditioning \
		 -deviceList $conditionList \
		 -system lts]
    set cld [APSMpConvertDetailedDeviceNames \
		 -operation StartConditioning \
		 -deviceList $cld \
		 -arrayName $arrayName]

    if {![info exists $arrayName] || !$configured} {
	APSMpGenericPSArraySetup -arrayName $arrayName \
	    -turnOnList $conditionList \
	    -turnOnListDetailed $cld \
	    -conditionList $conditionList \
	    -conditionListDetailed $cld \
	    -SCRFile $SCRFile \
	    -cycles $cycles \
	    -system lts
    }
    set conditionList [APSMpGenericPSArrayQuery \
			   -arrayName $arrayName \
			   -operation StartConditioning]
    set conditionList [APSMpConvertDeviceNames \
			   -operation StartConditioning \
			   -deviceList $conditionList \
			   -system lts]
    set conditionList [APSMpConvertDetailedDeviceNames \
			   -operation StartConditioning \
			   -deviceList $conditionList \
			   -arrayName $arrayName]
    
    set turnOnList $conditionList

    APSMpStep "Stop Conditioning"
    if [catch {APSDevSend -cavput 1 \
		   -releaseLinks 1 \
		   -timeout 0 \
		   -group PSOps \
		   -operation StopConditioning \
		   -deviceList $conditionList} result] {
	APSMpReturn error "APSMpLTS_ConditionPowerSupplies: $result : $conditionList"
    }	

    APSMpStep "Turn on needed supplies" -proc APSMpLTS_TurnOnPS
    if {[catch {APSMpLTS_TurnOnPS -turnOnList $turnOnList -configured 1} result]} {
	APSMpReturn error "APSMpLTS_ConditionPowerSupplies: $result"	    
    }
    set pass 0
    while {1} {
	incr pass
	if [catch {APSDevSend -releaseLinks 1 \
		       -timeout 10 \
		       -group PSOps \
		       -operation QueryOn \
		       -deviceList $conditionList} result] {
	    APSMpReturn error "APSMpLTS_ConditionPowerSupplies: $result"
	}
	set found 0
	foreach item $result device $conditionList {
	    if {!$item} {
		set found 1
		if {$pass == 8} {
		    if {[tk_dialog .warningDialog "Warning Message" \
			     "$device magnets are not on\nFix then click Continue" \
			     warning -1 Continue "Abort Procedure"] == 1} {
			APSMpReturn error "APSMpLTS_ConditionPowerSupplies: User aborted"
		    }
		}
	    }
	}
	if {$found == 0} {
	    break
	}
	APSWaitWithUpdate -waitSeconds 5	
    }    

    APSMpStep "Condition needed supplies" -proc APSMpLTS_ConditionPS
    if [catch {APSMpLTS_ConditionPS \
		   -arrayName $arrayName \
		   -configured 1 \
		   -SCRFile $APSMpLTSControlPS(Filename) } conditioning] {
	APSMpReturn error "APSMpLTS_ConditionPowerSupplies: $conditioning"
    }
    
    if $conditioning {
	APSMpLTS_WaitForConditioningPS -deviceList $conditionList 
    }
    
    set option ""
    set i 0
    foreach supply $conditionList {
	if {$i == 0} {
	    append option "-match=column,ControlName=${supply}*"
	    incr i
	} else {
	    append option ",ControlName=${supply}*,|"
	}
    }

    if [string length $APSMpLTSControlPS(Filename)] {
	APSMpStep RestoringSCRValues 
	set tmpFile /tmp/[APSTmpString]
	if [catch {exec sddsprocess $APSMpLTSControlPS(Filename) $tmpFile \
                     -match=column,IsReadOnly=+n \
                     -match=column,Category=MainPS,Category=SteeringPS,| \
                     -match=col,ControlType=,! -match=col,ControlName=,! $option
            exec sddscasr -restore $tmpFile} result] {
            catch {APSSendEMail -address soliday \
                     -subject "Problem restoring SCR file."  \
                     -message "Problem restore SCR file\nAPSMpLTS_ConditionPowerSupplies\n$SCRFile\n$result"} 
	    APSMpReturn error "Problem restoring SCR file: APSMpLTS_ConditionPowerSupplies: $SCRFile: $result"
	}
    }
    
    APSMpReturn ok
}

proc APSMpLTS_ConditionPS {args} {
    set arrayName APSMpLTSControlPS
    set configured 0
    set conditionList ""
    set conditionListDetailed ""
    set SCRFile ""
    APSStrictParseArguments {arrayName configured conditionList conditionListDetailed SCRFile}

    global $arrayName
    if {![info exists $arrayName] || !$configured} {
        APSMpGenericPSArraySetup -arrayName $arrayName \
	    -conditionList $conditionList \
	    -conditionListDetailed $conditionListDetailed \
	    -system lts
    }

    APSMpStep init -promptDialog \
      "APSMpGenericPSDialog -condition 1 -arrayName $arrayName -system LTS -frame "
    set conditionList [APSMpGenericPSArrayQuery \
			   -arrayName $arrayName \
			   -operation StartConditioning]
    set conditionList [APSMpConvertDeviceNames \
			   -operation StartConditioning \
			   -deviceList $conditionList \
			   -system lts]
    set conditionList [APSMpConvertDetailedDeviceNames \
			   -operation StartConditioning \
			   -deviceList $conditionList \
			   -arrayName $arrayName]

    if ![llength $conditionList] {
        APSMpReturn ok 0
    }

    APSMpStep "Configure conditioning"
    set cycles [set ${arrayName}(StartConditioningInfo,cycles)]

#	if [catch {exec standardize -configure /home/helios/oagData/controlFiles/LTS/LTS_standardize.std} result] {
#	    APSMpReturn error "APSMpLTS_ConditionPS: $result"
#	}
    if [catch {exec degauss -configure /home/helios/oagData/controlFiles/LTS/LTS_degauss.sdds} result] {
	APSMpReturn error "APSMpLTS_ConditionPS: $result"
    }
    if [catch {APSDevSend -releaseLinks 1 \
		   -group PSOps \
		   -operation SetConditioningCycles \
		   -deviceList $conditionList \
		   -value $cycles} result] {
	APSMpReturn error "APSMpLTS_ConditionPS: $result"
    }
    if {[string length $SCRFile] && [file exists $SCRFile]} {
        # extract the setpoints from the SCR file and use them as the 
        # conditioning endpoints
        APSMpStep "Configure conditioning from snapshot"

        set tmpRoot /tmp/[APSTmpString]
        if [catch {exec sddsprocess /home/helios/oagData/deviceConfig/lts/LTSOps.pvTable \
                     $tmpRoot.setpt -match=column,Operation=SetCurrent 
            exec sddsprocess /home/helios/oagData/deviceConfig/lts/LTSOps.pvTable \
                     $tmpRoot.setcond -match=column,Operation=SetConditioningEndpoint
            exec sddsxref -nowarning $SCRFile $tmpRoot.setpt -pipe=out \
                     -match=ControlName -take=DeviceName \
                     | sddsconvert -pipe -delete=column,ControlName \
                     | sddsxref -pipe=in $tmpRoot.setcond -nowarning \
                     $tmpRoot.snap -match=DeviceName -take=ControlName
            exec sddscasr -restore $tmpRoot.snap} result] {
            catch {eval file delete -force $tmpRoot.setpt $tmpRoot.setcond $tmpRoot.snap}
            APSMpReturn error "APSMpLTS_ConditionPS: $result"
        }
        catch {eval file delete -force $tmpRoot.setpt $tmpRoot.setcond $tmpRoot.snap}
    }

    APSMpStep "Starting conditioning for [llength $conditionList] devices" -desc "Starting conditioning"
    if [catch {APSDevSend -releaseLinks 1 \
		   -group PSOps \
		   -operation StartConditioning \
		   -deviceList $conditionList \
		   -timeout 0} result] {
        APSMpReturn error "APSMpLTS_ConditionPS: $result"
    }
    APSMpReturn ok [llength $conditionList]
}

proc APSMpLTS_WaitForConditioningPS {args} {
    set deviceList [APSDevSend -releaseLinks 1 \
			-group PSOps \
			-operation QueryConditioning]
    APSStrictParseArguments {deviceList}

    set timeToStop [expr [clock seconds]+20*60]
    set someConditioning 1
    APSMpStep "Waiting For Conditioning"
    set stepsTaken 0
    set query QueryConditioning
    while {($someConditioning) && ($timeToStop>[clock seconds])} {
        APSWaitWithUpdate -waitSeconds 10 -updateInterval 1
        set someConditioning 0
        incr stepsTaken
        if [catch {APSDevSend -releaseLinks 1 \
		       -timeout 10 \
		       -group PSOps \
		       -operation $query \
		       -deviceList $deviceList} result] {
            APSMpReturn error "APSMpLTS_WaitForConditionPS: $result"
        }
        set someConditioning 0
        foreach item $result {
            incr someConditioning $item
        }
        if $someConditioning {
            APSMpStep "Waiting For Conditioning of $someConditioning ganged devices"
            APSWaitWithUpdate -waitSeconds 15 -updateInterval 1
            if $stepsTaken==5 {
                APSMpStep "You are getting sleepy..."
            }
            if $stepsTaken==15 {
                APSMpStep "You are getting very sleepy..."
            }
            if $stepsTaken==20 {
                APSMpStep "You are getting very, very sleepy..."
            }
        }
    }
    if $someConditioning {
        if {![APSQueryToProceed -message "Some supplies still conditioning.  What do you want to do?"]} {
            APSMpReturn error "Some supplies did not finish conditioning."
        }
    }
    APSMpReturn ok
}
