# $Log: not supported by cvs2svn $
# Revision 1.39  2006/02/07 15:21:18  soliday
# Added code to check conditioning validity flags.
#
# Revision 1.38  2004/12/08 13:40:18  soliday
# Backed out of last change.
#
# Revision 1.37  2004/12/08 12:33:43  soliday
# Updated APSSrTurnOnDcps so that the proper error message is passed back
# if a power supply fails to turn on after multiple tries.
#
# Revision 1.36  2004/03/23 12:48:19  soliday
# Changed the BM off command.
#
# Revision 1.35  2004/03/01 21:24:17  soliday
# Updated so that it does not try to turn on canted undulators that do not exist.
#
# Revision 1.34  2004/02/23 16:21:42  soliday
# Fixed issues with turning off the canted undulators.
#
# Revision 1.33  2004/02/23 16:02:37  soliday
# Added code to start and condition the canted undulators.
#
# Revision 1.32  2004/02/11 16:10:05  soliday
# cavputs are now down for both <corr>:CurrentAO and SFB:<corr>:CurrentAO
#
# Revision 1.31  2002/10/04 00:02:38  emery
# Removed extraneous configFile in second APSParseArguments
# of APSSrDcpsAction
#
# Revision 1.30  2002/10/03 07:10:34  emery
# Fixed command for S:BM in SRDCPSAction procedure. There
# was an extra exec cavput in the variable command.
#
# Revision 1.29  2002/10/02 21:28:03  shang
# added prefix SFB: to correctors for setting corrector values.
#
# Revision 1.28  2002/10/01 14:07:48  soliday
# Changed the RAW PS back to OffBO
#
# Revision 1.27  2002/09/30 21:01:25  soliday
# The OffBO PVs no longer exist.
#
# Revision 1.26  2000/05/01 23:25:48  carwar
# Updated cavput for turn-on DCPS with pend=10 (originally default -pend)
#
# Revision 1.25  2000/01/19 06:20:07  emery
# In DCPS action for BM, filter the config file for
# S:BM, since the config file is no longer garanteed to have
# just one magnet.
#
# Revision 1.24  1999/01/19 14:41:47  borland
# Added some code to help deal with sectors lists that have blank elements.
#
# Revision 1.23  1998/11/02 23:51:15  borland
# From D. Blachowicz: now permits setting the number of standardization cycles.
#
# Revision 1.22  1998/06/15 16:35:55  borland
# Added ability to set tolerances for all PS.
#
# Revision 1.21  1998/05/04 20:11:07  emery
# Removed pause in between sending configuration commands
# for multiple sectors.
#
# Revision 1.20  1997/07/11 14:02:42  borland
# Now properly detects error exists from standardize program (e.g., for configure
# that fails due to missing device server).
#
# Revision 1.19  1996/12/09 21:07:16  borland
# Modified sector turn-on procedure to use new APSSrTurnOnDcps procedure.
#
# Revision 1.18  1996/12/09 20:58:33  borland
# Added smart turn-on procedure that only turns on those PS that are already
# off.  Also will make repeated turn-on attempts with a pause in between.
#
# Revision 1.17  1996/11/18 15:44:01  borland
# Fixed cavput value for turn-off of raw power supplies.
#
# Revision 1.16  1996/11/12 23:28:09  borland
# Fixed bugs and typos from previous work that was never tested.
#
# Revision 1.15  1996/10/05 03:45:13  borland
# Fixed test of status that bombed when ? was return.
#
# Revision 1.14  1996/10/05 02:14:32  borland
# Improved checking of status values for slow turn on.
#
# Revision 1.13  1996/10/02 16:39:36  borland
# Removed -errorinfo options from return's.  Just return messages as the
# value of the procedure.
#
# Revision 1.12  1996/09/30 20:55:24  borland
# Fixed some bugs in configuration code.
#
# Revision 1.11  1996/09/23 13:41:50  borland
# Fixed bug in skew quadrupole procedure.  Completed raw PS procedure.
#
# Revision 1.10  1996/09/21 23:29:01  borland
# Modified to use cavput rather than devSend.  Much simpler code.
#
# Revision 1.9  1996/03/28  23:08:32  borland
# Added procedure for slow sector turn on.  Not tested fully.
#
# Revision 1.8  1996/03/05  21:55:37  borland
# Now stagger sending of setpoints, off, and conditioning commands.
#
# Revision 1.7  1996/02/26  15:20:43  borland
# Fixed unescaped square brackets in Help routines.
#
# Revision 1.6  1996/01/26  15:56:25  borland
# Improved intelligence of what pauses are used when turning things on.
# Added scrolled status area.
#
# Revision 1.5  1996/01/25  21:28:35  borland
# Added a pause between even and odd sector on commands.
#
# Revision 1.4  1996/01/25  20:28:50  borland
# Added even/odd sector staggering to improve speed and lessen overload of
# IOCs.  Added code to recognized that all sectors are being sent to, and
# use optimized devSends.
#
# Revision 1.3  1996/01/24  18:29:41  borland
# Added check for a sector list that contains all sectors; if present, now
# goes global devSends rather than per-sector devSends.
#
# Revision 1.2  1996/01/17  16:58:54  saunders
# Added /usr/local/oag/lib_patch to front to auto_path.
#
# Revision 1.1  1996/01/16  21:49:07  saunders
# Added application specific tcl procedure libraries, which are installed
# as libraries, not applications.
#
#

set debug 0

#
# APSCheckStandardizeFile <filename>
#

proc APSCheckStandardizeFileHelp { } {
    global debug
    return "APSCheckStandardizeFile <filename>"
}

proc APSCheckStandardizeFile { filename } {
    global debug
    if {[string length $filename]==0 || ![file exists $filename]} { 
        return [APSErrorReturn 0 "$filename blank or nonexistent"] }
    set check [exec sddscheck $filename]
    if {$check!="ok"} { 
        return [APSErrorReturn 0 "$filename is not an SDDS file"] }

    lappend columnsExpected ControlName ControlType From To Minutes Seconds NumCycles \
      Approach Finish

    set columnsPresent [exec sddsquery -column $filename ]
    foreach expected $columnsExpected {
        if {[lsearch -exact $columnsPresent $expected]==-1} { 
            return [APSErrorReturn 0 "$filename is not a standardize file"] }
    }
    return 1
}


proc APSCheckDegaussFileHelp { } {
    global debug
    return "APSCheckDegaussFile <filename>"
}

#
# APSCheckDegaussFile <filename>
#

proc APSCheckDegaussFile { filename } {
    global debug
    if {[string length $filename]==0 || ![file exists $filename]} {
        return [APSErrorReturn 0 "$filename blank or nonexistent"] }
    set check [exec sddscheck $filename]
    if {$check!="ok"} {
        return [APSErrorReturn 0 "$filename is not an SDDS file"] }

    lappend columnsExpected ControlName ControlType DecayMinutes DecaySeconds MaxCurrent \
      NumDecay PeriodSeconds

    set columnsPresent [exec sddsquery -column $filename ]
    foreach expected $columnsExpected {
        if {[lsearch -exact $columnsPresent $expected]==-1} { 
            return [APSErrorReturn 0 "$filename is not a degauss file"] }
    }
    return 1
}

proc APSSrTurnOnDcpsHelp {} {
    return "APSSrTurnOnDcps -family {Q S H V QS BM MT QS4 C:BM C:H C:V} \
 \[-sectors <sector-list>\] \[-pause <millsec>\] \[-tries <number>\]"
}


proc APSSrTurnOnDcps {args} {
    global debug
    set family ""
    set sectors ""
    set statusCallback ""
    set tries 1
    set pause 5000
    APSParseArguments {family sectors statusCallback tries pause}

    if {([string compare $family "C:BM"] == 0) || 
        ([string compare $family "C:H"] == 0) || 
        ([string compare $family "C:V"] == 0)} {
        if [llength $sectors]==0 {
            for {set i 1} {$i <= 40} {incr i} {
                append sectors "$i "
            }
        }
        set sectorsList $sectors
        if {[catch {exec sdds2stream \
		      -col=Sector,CUflag,CUdipoleExists /home/helios/oagData/sr/XAXSTF/sectors.sdds} results]} {
            return -code error  "APSSrDcpsAction: $results"
        } else {
            set sectorsList ""
            foreach "sector flag bmFlag" $results {
                if {([string compare $family "C:BM"] == 0) && ($bmFlag == 0)} {
                    continue
                }
                if {$flag} {
                    if {[lsearch -exact $sectors $sector] != -1} {
                        lappend sectorsList $sector
                    }
                }
            }
            if {![llength $sectorsList]} {
                return
            }
        }
        set sectors $sectorsList
    }
    
    if [llength $sectors]==0 {
        lappend lists -list=S -range=begin=1,end=40
    } else {
        lappend lists -list=S -list=[join $sectors ,]
    }

    switch $family {
        "QS" { 
            return [APSSrSkewAction -action on -sectors $sectors -pause $pause]
        }
        "BM" { 
            set lists -list=S:BM
        }
        "C:BM"  { 
            lappend lists -list=C:BM
        }
        "Q"  { 
            lappend lists -list=A:Q,B:Q -range=begin=1,end=5
        }
        "C:V"  { 
            lappend lists -list=C:V1
        }
        "C:H"  { 
            lappend lists -list=C:H1
        }
        "V"  { 
            lappend lists -list=A:V,B:V -range=begin=1,end=4
        }
        "H"  { 
            lappend lists -list=A:H,B:H -range=begin=1,end=4
        }
        "S"  { 
            lappend lists -list=A:S1,A:S2,A:S3,A:S4,B:S3,B:S2,B:S1
        }
        "MT" { 
            lappend lists -list=A:MT,B:MT
        }
        "QS4" {
            lappend lists -list=A:QS4
        }
        default { 
            return -code error  "APSSrTurnOnDcps: Invalid family $family."  
        }
    }
    
    while {$tries} {
        incr tries -1
        if [catch {eval APScavget $lists -list=:StatusCALC -pend=30 -label} result] {
            return -code error "APSSrTurnOnDcps: $result"
        }
        set statusList [APSSearchAssociateListForValue -list $result -notvalue 0]
        if ![llength $statusList] {
            return 
        }
        regsub -all {:StatusCALC} $statusList {} statusList
        if [catch {eval APScavput -pend=10 -blunder=silent -list=[join $statusList ,] -list=:ResetSEQ=1} result] {
            return -code error "APSSrTurnOnDcps: $result"
        }
        if $tries {
            after $pause
        }
    }
}

proc APSSrDcpsActionHelp {} {
    global debug
    return "APSSrDcpsAction -family {Q S H V QS BM MT QS4 C:BM C:H C:V} -action {on off configure condition}\
 \[-value <value>\] \[-configFile <file>\] \[-sectors <sector-list>\] \[-pause <millsec>\]"
}

proc APSSrDcpsAction {args} {
    global debug
    set family ""
    set action ""
    set configFile ""
    APSParseArguments {family action configFile}

    if {$family=="" || $action==""} {
        return -code error  "-family and -action must be supplied."
    }
    if {$action=="on"} {
        return [eval APSSrTurnOnDcps -family $family $args]
    }

    if [lsearch -exact [list Q S BM H V QS QS4 MT C:BM C:H C:V] $family]==-1 {
        return -code error  "unknown family: $family"
    }

    if [lsearch -exact [list configure standardize degauss] $action]!=-1 {
        if {$configFile==""} { 
            return -code error  "-configFile must be given."
        }
        switch $family {
            Q -
            S -
            BM -
            C:BM {
                if {![APSCheckStandardizeFile $configFile]} {
                    return -code error  "$configFile nonexistent or invalid."
                }
            }
            H -
            V -
            QS -
            C:H -
            C:V {
                if {![APSCheckDegaussFile $configFile]} {
                    return -code error  "$configFile nonexistent or invalid."
                }
            }
            default {
                return -code error  "Can't condition family $family" 
            }
        }
    }

    if {($family=="QS") && ($action != "configure")} {
        return [eval APSSrSkewAction -action $action -configFile $configFile $args]
    }
    
    set value ""
    set sectors ""
    set statusCallback ""
    set pause 500
    APSParseArguments {value sectors statusCallback pause}
    APSClearOptions
    
    if {([string compare $family "C:BM"] == 0) || 
        ([string compare $family "C:H"] == 0) || 
        ([string compare $family "C:V"] == 0)} {
        if [llength $sectors]==0 {
            for {set i 1} {$i <= 40} {incr i} {
                append sectors "$i "
            }
        }
        set sectorsList $sectors
        if {[catch {exec sdds2stream \
		      -col=Sector,CUflag,CUdipoleExists /home/helios/oagData/sr/XAXSTF/sectors.sdds} results]} {
            return -code error  "APSSrDcpsAction: $results"
        } else {
            set sectorsList ""
            foreach "sector flag bmFlag" $results {
                if {[string compare $family "C:BM"] == 0} {
                    if {$bmFlag} {
                        if {[lsearch -exact $sectors $sector] != -1} {
                            lappend sectorsList $sector
                        }
                    }
                } else {
                    if {$flag} {
                        if {[lsearch -exact $sectors $sector] != -1} {
                            lappend sectorsList $sector
                        }
                    }
                }
            }
            if {![llength $sectorsList]} {
                return
            }
        }
        set sectors $sectorsList
    }
    lappend command -pend=15 -list=S
    if [llength $sectors]==0 {
        lappend command -range=begin=1,end=40
    } else {
        lappend command -list=[join $sectors ,]
    }

    set matchString \[AB\]:$family\[0-9\]*
    set configCommand standardize
    set prefix ""
    switch $family {
        "QS" { 
            #return -code error  "APSSrDcpsAction: invalid family in switch statement: QS"
            set matchString \[AB\]:QS
            set configCommand degauss
        }
        "BM" { 
            set matchString S:BM*
            set command ""
            lappend command -pend=10 -list=S:BM
        }
        "C:BM"  { 
            set matchString C:BM*
            lappend command -pend=10 -list=C:BM
        }
        "Q"  { 
            lappend command -list=A:Q,B:Q -range=begin=1,end=5
        }
        "C:V"  { 
            set matchString C:V1*
            lappend command -pend=10 -list=C:V1
            set configCommand degauss
        }
        "C:H"  { 
            set matchString C:H1*
            lappend command -pend=10 -list=C:H1
            set configCommand degauss
        }
        "V"  { 
            lappend command -list=A:V,B:V -range=begin=1,end=4 
            set configCommand degauss
        }
        "H"  { 
            lappend command -list=A:H,B:H -range=begin=1,end=4 
            set configCommand degauss
        }
        "S"  { 
            lappend command -list=A:S1,A:S2,A:S3,A:S4,B:S3,B:S2,B:S1
        }
        "MT" { 
            lappend command -list=A:MT,B:MT
        }
        "QS4" {
            #QS4 replaced A:V4 but B:V4 is still there
            lappend command -list=A:QS4
        }
        default { 
            return -code error  "Invalid family $family."  
        }
    }

    switch -exact $action {
        set {
            if {[string compare $family "H"]==0 || [string compare $family "V"]==0} {
                set prefix "-list=,SFB:"
            }
            lappend command -list=:CurrentAO=$value
        }
        setTol {
            lappend command -list=:ToleranceCALC.C=$value
        }
        on { 
            lappend command -list=:ResetSEQ=1
        }
        off { 
            if {[string compare $family "C:BM"]==0} {
                lappend command -list=:OffSEQ.PROC=1
            } elseif {[string compare $family "BM"]==0} {
                lappend command -list=:OffBO=1
            } else {
                lappend command -list=:OnBO=0
            }
        }
        forcecondition -
        forcestandardize -
        forcedegauss { 
            lappend command -list=:StandardizeSUB.PROC=1
        }
        condition -
        standardize -
        degauss { 
            switch $family {
                Q -
                S -
                BM -
                C:BM {
                    set checkcommand "$command -list=:CnditValCC,:StandardizeSUB.G,:StandardizeSUB.H,:CurrentAO -label"
                    set command ""
                    set command2 ""
                    set command3 ""
                    if [catch {eval exec cavget $checkcommand} result] {
                        return -code error  "APSSrDcpsAction1: $result"
                    }
                    foreach "cnditName cnditVal finishName finishVal approachName approachVal currentName currentVal" $result {
                        if {($cnditVal == 1) && ((($approachVal == 0) && ($finishVal >= $currentVal)) || 
                                                 (($approachVal == 1) && ($finishVal <= $currentVal)))} {
                            #If no conditioning is needed just set the final value. This way
                            #the end result will be exactly the same as if conditioning was done.
                            lappend command2 ${currentName}=$finishVal
                            #if {[catch {APScavput -pend=15 -blunderAhead=silently -list=${currentName}=$finishVal} result]} {
                            #    return -code error "APSSrDcpsAction2: $result"
                            #}
                        } elseif {$cnditVal == 1} {
                            #set cycles to zero
                            set basename [string range $cnditName 0 [expr {[string last : $cnditName] - 1}]]
                            lappend command3 ${basename}:StandardizeSUB.F=0
                            #if {[catch {exec cavput -pend=15 -blunderAhead=silently -list=${basename}:StandardizeSUB.F=0} result]} {
                             #   return -code error "APSSrDcpsAction: $result" 
                            #}
                            lappend command "[string range $cnditName 0 [expr {[string last : $cnditName] - 1}]]"
                        } else {
                            lappend command "[string range $cnditName 0 [expr {[string last : $cnditName] - 1}]]"
                        }
                    }
                    if {[llength $command2]} {
                        if {[catch {exec cavput -pend=15 -blunderAhead=silently -list=[join $command2 ,]} result]} {
                            return -code error "APSSrDcpsAction: $result"
                        }
                    }
                    if {[llength $command3]} {
                        if {[catch {exec cavput -pend=15 -blunderAhead=silently -list=[join $command3 ,]} result]} {
                            return -code error "APSSrDcpsAction: $result"
                        }
                        after 1000
                    }
                    if {[llength $command]} {
                        set command "-pend=15 -blunderAhead=silently -list=[join $command ,]"
                    } else {
                        return -code ok
                    }
                }
            }

            lappend command -list=:StandardizeSUB.PROC=1
        }
        stopcond {
            lappend command -blunderAhead=silently -list=:StandardizeSUB.PROC=0
        }
        setcycles {
            lappend command -blunderAhead=silently -list=:StandardizeSUB.F=$value
        }    
        configure {
            set configFile1 [APSTmpDir]/[APSTmpString]
            if [string compare $family BM]==0 {
                exec sddsprocess $configFile $configFile1 \
                  "-match=column,ControlName=S:BM*" -nowarnings
                if {[exec sdds2stream -rows $configFile1]=="0 rows"} {
                    return -code error  "No configuration data for S:BM"
                }
                if {$statusCallback!=""} {
                    $statusCallback "Configuring S:BM"
                }
                if [catch {exec $configCommand -configure $configFile1} result] {
                    return -code error $result
                }
            } else {
                foreach sector [split [string trim $sectors]] {
                    if ![string length $sector] continue
                    exec sddsprocess $configFile $configFile1 \
                      "-match=column,ControlName=S${sector}$matchString" -nowarnings
                    if {[exec sdds2stream -rows $configFile1]=="0 rows"} {
                        if {$family == "QS"} {
                            continue
                        }
                        return -code error  "No configuration data for S${sector}:$family"
                    }
                    if {$statusCallback!=""} {
                        $statusCallback "Configuring S${sector} family $family"
                    }
                    if [catch {exec $configCommand -configure $configFile1} result] {
                        if {$family != "QS"} {
                            return -code error $result
                        }
                    }
                }
            }
            return -code ok
        }
    }
    if $debug {
        puts stderr "$command"
        return -code ok
    } 
#    puts "\n\nexec cavput $prefix $command"
    if [catch {eval exec cavput $prefix $command} result] {
        return -code error  "APSSrDcpsAction3: $result"
    }
    if {0} {
    if {$family=="QS4"} {
        #need do the same thing for B:V4 at the same sectors
        set command [regsub -all {A:QS4} $command "B:V4"]
        if [catch {eval exec cavput -list=SFB: $command } result] {
            return -code error  "APSSrDcpsAction4: $result"
        }
    }
    }
    return -code ok
}

proc APSSetSrDcpsFamilyHelp {} {
    global debug
    return "APSSetSrDcpsFamily -family {Q S QS BM H V MT QS4 C:BM C:H C:V} -value <value> \[-sectors <sectors>\] \[-pause <millsec>\]\n\
Default is all sectors."
}

proc APSSetSrDcpsFamily {args} {
    return [eval APSSrDcpsAction -action set $args]
}


proc APSSrSkewAction {args} {
    global debug
    #        puts -nonewline "APSSrSkewAction: "
    #        foreach arg $args {puts -nonewline "$arg "}
    #        puts ""

    global apsSrSkewName
    if {![info exists apsSrSkewName]} { APSMakeSrSkewNames }

    set action ""
    set value ""
    set configFile ""
    set sectors ""
    set pause 250
    APSParseArguments {action value configFile sectors pause}
    APSClearOptions

    set skewList ""
    if [llength $sectors]==0 {
        for {set sector 1} {$sector<40} {incr sector 2} {
            lappend skewList $apsSrSkewName($sector)
            lappend sectorList $sector
        }
    } else {
        foreach sector $sectors {
            if {$sector>0 && $sector<40 && [expr $sector%2]} {
                lappend skewList $apsSrSkewName($sector)
                lappend sectorList $sector
            }
        }
    }       
    if {![llength $sectorList]} { 
        return -code error  "No valid sectors given for skew quads"
    }

    lappend command APScavput -pend=15 -blunderAhead=silently -list=[join $skewList ,]
    switch $action {
        set {
            lappend command -list=:CurrentAO=$value
        }
        setTol {
            lappend command -list=:ToleranceCALC.C=$value
        }
        on { 
            lappend command -list=:ResetSEQ=1
        }
        off { 
            lappend command -list=:OnBO=0
        }
        forcecondition -
        forcestandardize -
        forcedegauss - 
	condition -
	standardize -
	degauss { 
	    lappend command -list=:StandardizeSUB.PROC=1
	}
	stopcond {
	    lappend command -list=:StandardizeSUB.PROC=0
	}
	configure {
	    if {$configFile==""} { 
		return -code error  "No configuration file given for skew quads"
	    }
	}
	default { 
	    return -code error  "Invalid skew quad action"
	}
    }
    if $debug {
	puts stderr "$command"
	return -code ok
    }
    if [catch $command result] {
	return -code error  "APSSrDcpsAction: $result"
    }
    return -code ok 
}

proc APSMakeSrSkewNames {} {
    global debug
    global apsSrSkewName
    for {set sector 1} {$sector<41} {incr sector} {
	set apsSrSkewName($sector) ""
    }
    foreach sector {3 7 11 15 19 23 27 31 35 39} {
	set apsSrSkewName($sector) "S${sector}A:QS"
    }
    foreach sector {1 5  9 13 17 21 25 29 33 37} {
	set apsSrSkewName($sector) "S${sector}B:QS"
    }
}


proc APSSrRpsAction {args} {
    global debug
    set action ""
    set group ""
    set sectors ""
    APSParseArguments {action group sectors}
    if [string compare $action ""]==0 {
	return -code error  "No action supplied for RPS."
    }
    if [lsearch -exact [list on off] $action]==-1 {
	return -code error  "Unknown raw PS action: $action"
    }
    if [lsearch -exact [list R1 R2 R3 R4 R] $group]==-1 {
	return -code error  "No valid group supplied for RPS."
    }
    
    if [llength $sectors]==0 {
	set sectorList {}
	for {set sector 1} {$sector<40} {incr sector 2} {
	    lappend sectorList $sector
	}
    } else {
	foreach sector $sectors {
	    if {$sector>0 && $sector<40} {
		lappend sectorList $sector
	    }
	}
    }
    if {![llength $sectorList]} { 
	return -code error  "No valid sectors given for RPS"
    }
    
    set prefixList {}
    foreach sector $sectorList {
	if {$sector<0 || $sector>40 || [expr $sector%2]==0} { 
	    continue
	}
	set other [expr $sector-1]
	if {$other==0} { set other 40 }
	lappend prefixList S${other}:${sector}:
    }
    
    lappend command APScavput -blunderAhead=silently -pend=15 -list=[join $prefixList ,]
    
    switch $group {
	R1 -
	R2 -
	R3 -
	R4 {
	    lappend command -list=$group
	}
	R {
	    lappend command -list=R1,R2,R3,R4
	}
    }

    switch $action {
	on {
	    lappend command -list=:ResetSEQ=1,:ONSEQ=1
	}
	off {
	    lappend command -list=:OffBO=1,:OFFSEQ=1
	}
    }
    if $debug {
	puts stderr "$command"
	return -code ok
    }
    if [catch $command result] {
	return -code error  "APSSrRpsAction: $result"
    }
    return -code ok
}


proc SRTurnOnSectorHelp {} {
    return "SRTurnOnSector -family {Q S H V} -sector <number> \[-pause <millisec>\] \[-repeats <integer>\] \[-statusCallback <procName>\]"
}

# this routine is basically obsolete
proc SRTurnOnSector {args} {
    set family "?"
    set sector 0
    set pause 10000
    set repeats 1
    set statusCallback ""
    if [APSStrictParseArguments {family sector pause repeats statusCallback}]==-1 {
	return -code error  "Bad arguments to SRTurnOnSector"
    }
    if [lsearch -exact {Q S H V} $family]==-1 {
	return -code error  "SRTurnOnSector: Unknown family: $family"
    }
    if {$sector<0 || $sector>40} {
	return -code error  "SRTurnOnSector: Bad sector: $sector"
    }
    if $repeats<=0 {
	return -code error  "SRTurnOnSector: Bad repeats: $repeats"
    }
    if $pause<0 {
	return -code error  "SRTurnOnSector: Bad pause: $pause"
    }
    if $pause<1000 {
	set pause 1000
	if [string length $statusCallback] {
	    $statusCallback "Pause too small--set to 1000 milliseconds"
	}
    }
    if [catch {eval APSSrTurnOnDcps -family $family -sectors $sector -pause $pause -tries $repeats \
		 -statusCallback $statusCallback} result] {
	return -code error "SRTurnOnSector: $result"
    }
}

proc MakeSectorList {SectorSelectionList} {
    global debug
    array set SectorSelection $SectorSelectionList
    set sectors {}
    foreach sector [array names SectorSelection] {
	if {$sector>40 || $sector<0} { continue }
	if {$SectorSelection($sector)} {
	    lappend sectors $sector
	}
    }
    return [lsort -integer $sectors]
}
