/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

/*
 * $Log: not supported by cvs2svn $
 *
 */

#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include <tclRPN.h>

#define VERSION "1.0"

#if ((TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION >= 4)) || (TCL_MAJOR_VERSION > 8)
#define RPNCONST const
#else
#define RPNCONST
#endif

static Tcl_HashTable tclRPNCmdTable;

typedef struct {
  char *name;     /* rpn xxx command name */
  int (*proc)();  /* command handling function */
} CMD_INFO;

static CMD_INFO tclRPNCommands[] = {
  {"expr", tclRPN_Expr},
  {"clear", tclRPN_Clear},
  {"if2pf", tclRPN_if2pf},
  {(char *)NULL, (int (*)())NULL}
};


/* Function Prototypes */
EXTERN int Tclrpn_Init(Tcl_Interp *interp);
int tclRPNCmd(ClientData dummy, Tcl_Interp *interp, int argc, RPNCONST char *argv[]);
int tclRPNCmd(ClientData dummy, Tcl_Interp *interp, int argc, RPNCONST char *argv[])
{

  Tcl_HashEntry *entryPtr;
  int (*handler)();

  if (argc < 2) {
    Tcl_SetResult(interp, "rpn: must specify a sub-command", 
		  (Tcl_FreeProc *) NULL); 
    return TCL_ERROR;
  }

  entryPtr = Tcl_FindHashEntry(&tclRPNCmdTable, argv[1]);
  if (entryPtr == NULL) {
    Tcl_SetResult(interp, "Unknown rpn command ", (Tcl_FreeProc *) NULL);
    return TCL_ERROR;
  } else {
    handler = (int(*)())Tcl_GetHashValue(entryPtr);
    return ((*handler)(interp, argc, argv));
  }

}

int Tclrpn_Init(Tcl_Interp *interp) {
  CMD_INFO *cmd;
  if (Tcl_InitStubs(interp, "8.0", 0) == NULL) {
    return TCL_ERROR;
  }
  if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 0) == NULL) {
    if (TCL_VERSION[0] == '7') {
      if (Tcl_PkgRequire(interp, "Tcl", "8.0", 0) == NULL) {
	return TCL_ERROR;
      }
    }
  }
  if (Tcl_PkgProvide(interp, "rpn", VERSION) != TCL_OK) {
    return TCL_ERROR;
  }

  Tcl_CreateCommand(interp, "rpn", tclRPNCmd, (ClientData *) NULL,
		    (Tcl_CmdDeleteProc *)NULL);

  Tcl_InitHashTable(&tclRPNCmdTable, TCL_STRING_KEYS);
  for (cmd=tclRPNCommands ; cmd->name != NULL ; cmd++) {
    int newEntry;
    Tcl_HashEntry *entryPtr;
    entryPtr = Tcl_CreateHashEntry(&tclRPNCmdTable, cmd->name, &newEntry);
    Tcl_SetHashValue(entryPtr, cmd->proc);
  }
  tclRPNInit(interp);
  
  return TCL_OK;
}




