
# $Log: not supported by cvs2svn $
# Revision 1.47  2011/10/17 20:33:19  shang
# updated APSSetTuneMultiplexer due to hardware changes.
#
# Revision 1.46  2011/10/12 03:19:41  lemery
# Changed PV SR:TUNE:HP_1366A_C1P1_MBBO to SR:TUNE:HP_1366A_C2P1_MBBO.
#
# Revision 1.45  2011/09/23 21:56:06  shang
# fixed a typo
#
# Revision 1.44  2011/09/23 21:53:06  shang
# updated APSSetTuneMultiplexer due to hardware change.
#
# Revision 1.43  2010/06/01 21:18:05  shang
# modified to set the drive to OFF state for cavity/sum signal.
#
# Revision 1.42  2010/06/01 21:08:00  shang
# modified  APSSetTuneMultiplexer to setup multiplexer for "Cavity" and "Sum" modes
#
# Revision 1.41  2010/04/05 21:36:12  shang
# removed PTBBPM since the bpms are included in LPL already.
#
# Revision 1.40  2010/04/05 19:29:05  shang
# added PTB BPMs
#
# Revision 1.39  2010/03/16 17:41:06  shang
# corrected error messages.
#
# Revision 1.38  2010/03/12 15:15:27  shang
# modified  APSSetTuneMultiplexer to use the new mux pv.
#
# Revision 1.37  2010/02/25 22:07:37  shang
# added StreakCamera to LPL
#
# Revision 1.36  2010/02/12 22:23:11  shang
# added PARBPM to LPL system
#
# Revision 1.35  2010/01/04 18:25:22  shang
# added description for P0Feedback
#
# Revision 1.34  2010/01/04 18:17:15  shang
# added P0Feedback to SR
#
# Revision 1.33  2009/06/02 23:55:04  emery
# Indented procedure APSSetTuneMultiplexer
#
# Revision 1.32  2009/05/22 16:28:55  shang
# modified to use SR:TUNE:Mux4MO to switch input mux from 1366 to 1368
#
# Revision 1.31  2009/05/21 16:31:50  shang
# removed setting the HP_1366A switch in APSSetTunMultiplexer since it is being set when the stripline is set.
#
# Revision 1.30  2009/05/21 16:20:26  shang
# changed cavput -list=SR:TUNE:HP_1366A_P0_MBBO=ON to cavput -list=SR:TUNE:HP_1366A_P0_MBBO=0 since the former fails.
#
# Revision 1.29  2009/05/20 16:33:57  shang
# added setting SR:TUNE:HP_1366A_P1_MBBO to 1 for NASA or 2 for VSA in APSSetTuneMultiplexer
#
# Revision 1.28  2009/03/27 15:04:33  shang
# moved the APSSetupStriplineDrive procedure to APSMpPARMeasureTunes.tcl
#
# Revision 1.27  2009/03/27 14:35:10  shang
# added  APSSetupStriplineDrive procedure
#
# Revision 1.26  2009/03/18 20:41:27  shang
# added SBPMWaveform to SR system
#
# Revision 1.25  2008/06/12 20:40:41  shang
# added BoosterTune to "Booster Systems"
#
# Revision 1.24  2008/06/03 19:50:49  emery
# Added setting SR:TUNE:HP_1366A_P0_MBBO in APSSetTuneMultiplexer
#
# Revision 1.23  2008/03/11 01:11:22  emery
# Replaced "wr" questionable voltage codes for custom Xray bpm compare with
# "lowV". Also use two columns State1 and State2 from the beginning
# of processing.
#
# Revision 1.22  2008/02/27 00:38:27  emery
# Previously APSSRXrayBPMCustomCompare compared only the downstream
# gap (variable gap). Now the two gaps are compared. If one is
# out of tolerance, then sector is rejected.
#
# Revision 1.21  2008/01/29 16:42:08  soliday
# Fixed last change.
#
# Revision 1.20  2008/01/29 16:34:37  soliday
# Added the IDEncoder group to SCR per Marty Smith's request.
#
# Revision 1.19  2006/10/17 22:11:05  shang
# added display of USgap, removed gapThreshold filter for compare SRXrayBPM and modified to display
# all ID gaps including those that do not have ds gap.
#
# Revision 1.18  2006/07/07 16:09:18  soliday
# Changed LINAC/PAR/LEUTL label to LINAC/PAR/LET
#
# Revision 1.17  2004/10/26 19:45:55  soliday
# Fixed a typo in a SCR system description.
#
# Revision 1.16  2004/10/26 19:43:19  soliday
# Added the ability to setup the SCR systems using an SDDS file.
#
# Revision 1.15  2004/06/08 16:25:19  emery
# Fixed spelling in one error message.
#
# Revision 1.14  2004/06/03 15:52:18  borland
# Added custom review feature, plus custom review choices for SR and SBPMs.
#
# Revision 1.13  2004/04/21 20:24:26  soliday
# While I don't like code outside of a procedure, especially in a library,
# and that can potentially cause a program to exit, I have only applied a bandaid.
# The code to call APSSCRDefineVariables can be avoided by defining
# apsSkipSCRDefineVariables. Also the method for displaying the error message
# has been changed. It looked to see if tk_version was defined but when apsHelp
# sources this from inside a function it did not check inside the global namespace.
#
# Revision 1.12  2004/03/17 16:51:15  shang
# added apsSCRFirstSnapshot and apsSCRSnapshotLogFile variable
#
# Revision 1.11  2004/03/16 19:56:35  shang
# set the apsSCRAccelManagerAccount variable for LPL,LTS and MPS system
#
# Revision 1.10  2004/03/05 21:30:44  shang
# set the apsSCRCustomCompareList(SRXrayBPM) and apsSCRCustomCompareDescription(SRXrayBPM)
# variable for custom-compare of SRXrayBPM system; added APSSCRProcessXrayBPMSnapshot,
# APSSRXrayBPMCustomCompare, and APSSCRPrintXrayBPMCompTable procedures for SRXrayBPM
# custom compare.
#
# Revision 1.9  2004/03/01 16:57:40  shang
# added SRXrayBPM to SR System
#
# Revision 1.8  2004/02/23 15:41:00  shang
# set apsSCRAtticDir variable to be global
#
# Revision 1.7  2004/02/23 15:36:50  shang
# added apsSCRAtticDir variable
#
# Revision 1.6  2004/02/18 16:46:41  shang
# added apsSCRUseCASRServer global variable
#
# Revision 1.5  2004/02/03 01:09:28  borland
# Handles the C0 correctors now.
#
# Revision 1.4  2003/10/22 15:32:45  soliday
# Re-added call to APSSCRDefineVariables. An alert box will display any
# error messages if they occur. This would happen at other sites if they
# have not customized the SCR directories in the procedure.
#
# Revision 1.3  2003/10/22 13:46:54  borland
# Removed call to APSSCRDefineVariables from APSStandardSetup.
# This call is now executed whenever SCRprocedures.tcl is loaded.
# Modified APSSCRDefineVariables (in SiteSpecificSCR.tcl) so that
# it doesn't do anything except on the first call.
#
# Revision 1.2  2003/07/20 18:15:43  borland
# Defined new global variable apsSCRPreSelectedSystem, used in
# SaveCompareRestore.
#
# Revision 1.1  2003/07/18 00:40:03  borland
# Code that was originally in SCRprocedures.tcl, but that was site-specific
# to APS.  Separating this code out allows other sites to use OAG SCR
# without modifying SCRprocedures.tcl.  They just make their own version
# of this file.
#


#
# These procedures used to be in SCRprocedures.tcl.
# However, they are site-specific and where moved here to allow generalizing
# SCR.

#
# List of procedures:
#
# This one must be defined for all sites:
# APSSCRDefineVariables : defines global variables needed for using SCR procedures
#         executed when this file is loaded.
#
# These are APS-specific:
# APSSCRSRCompareConvertors : does custom SR comparisons for convertors (power supplies)
# APSSCRSRCompareOrbitSetpoints : does custom SR comparisons for orbit setpoints
# APSSCRSRCompareOrbitOffsets : does custom SR comparisons for orbit offsets
# APSSCRSRCompareBPMQuantities : does custom SR comparisons for BPM quantities.
#  Used by APSSCRSRCompareOrbitSetpoints and APSSCRSRCompareOrbitOffsets.
# APSSetTuneMultiplexer: sets SR:TUNE:Mux4Sw2BO to NASA or VSA and
#   SR:TUNE:Mux4MO in both mode

proc APSSCRDefineVariables {args} {
    set topDirectory ""
    set inhibitLogging 0

    APSStrictParseArguments {topDirectory inhibitLogging}

    if ![string length $topDirectory] {
        set topDirectory /home/helios/oagData/SCR
    }
 
    global apsSCRDefineVariablesCalled
    if {[info exists apsSCRDefineVariablesCalled]  && \
          $apsSCRDefineVariablesCalled} {
        return 
    }
    set apsSCRDefineVariablesCalled 1
    
    global apsSCRRequestDir apsSCRSnapDir apsSCRInhibitActionLogging apsSCRAtticDir
    global apsSCRFilterFileDir apsSCRFilterFileName
    global apsSCRUseCarestore apsSCRUseGPServer apsSCRPVNameFilter apsSCRUseCASRServer
    global apsSCRSystemDescription apsSCRAccelManagerAccount
    global apsSCRSystemList apsSCRSystem apsSCRRefChoiceDir apsSCRCustomCompareList
    global apsSCRCustomCompareDescription apsSCRTopDirectory
    global apsSCRCustomReviewList apsSCRCustomReviewDescription
    global apsSCRGroupMembers apsSCRGroupDescription apsSCRGroup
    global apsSCRGroupMenuOrder apsSCRInvertFilterFile
    global apsSCRSuppressSingleBeamline apsSCRPreSelectedSystem
    global apsSCRFirstSnapshot apsSCRSnapshotLogFile

    set apsSCRTopDirectory $topDirectory
    set apsSCRRequestDir $topDirectory/requestFiles
    set apsSCRSnapDir $topDirectory/snapshots
    set apsSCRAtticDir $topDirectory/attic
    set apsSCRFilterFileDir $topDirectory/filterFiles/
    set apsSCRRefChoiceDir $topDirectory/preferredChoices
    set apsSCRFilterFileName ""
    set apsSCRUseCarestore 0
    set apsSCRUseGPServer 0
    set apsSCRUseCASRServer 0
    set apsSCRPVNameFilter *
    set apsSCRInhibitActionLogging $inhibitLogging
    set apsSCRInvertFilterFile 0
    set apsSCRSuppressSingleBeamline 0
    set apsSCRPreSelectedSystem ""
    
    if {[file exists ${topDirectory}/SCR.spec]} {
        if {[catch {sdds load ${topDirectory}/SCR.spec spec} result]} {
            return -code error "Unable to load ${topDirectory}/SCR.spec"
        }
        foreach groupDesc $spec(Parameter.GroupDescription) \
          group $spec(Parameter.Group) \
          systems $spec(Column.System) \
          descs $spec(Column.Description) \
          managers $spec(Column.Manager) {
              set apsSCRGroupDescription($group) $groupDesc
              lappend apsSCRGroupMenuOrder $group
              set apsSCRGroupMembers($group) $systems
              foreach system $systems desc $descs manager $managers {
                  set apsSCRSystemDescription($system) $desc
                  set apsSCRAccelManagerAccount($system) $manager
              }
          }
    } else {
        set apsSCRSystemDescription(PLET) "PAR/LET"
        set apsSCRSystemDescription(SR)  "Storage ring"
        set apsSCRSystemDescription(Booster) "Booster"
        set apsSCRSystemDescription(BRF) "Booster RF"
        set apsSCRSystemDescription(LINAC) "LINAC"
        set apsSCRSystemDescription(LEUTL) "LEUTL"
        set apsSCRSystemDescription(SRF) "Storage ring RF"
        set apsSCRSystemDescription(SBPMs) "Storage ring BPM controls"
        set apsSCRSystemDescription(SBPMLibera) "Storage ring Libra BPM controls"
        set apsSCRSystemDescription(SBPMWaveform) "Storage ring BSP-100 BPM timing controls"
        set apsSCRSystemDescription(SBPMLiberaWaveform) "Storage ring Libra BPM timing controls"
        set apsSCRSystemDescription(P0Feedback) "Storage ring P0 feedback system controls"
        set apsSCRSystemDescription(TFeedback) "Storage ring Transverse feedback system controls"
        set apsSCRSystemDescription(XrayTranslation) "Portable Xray BPM Translation System"
        set apsSCRSystemDescription(MPS) "Storage ring Machine Protection System"
        set apsSCRSystemDescription(SRNASA) "SR Network/Spectrum Analyzer"
	set apsSCRSystemDescription(PAR) "PAR"
        set apsSCRSystemDescription(PARNASA) "PAR Network/Spectrum Analyzer"
	set apsSCRSystemDescription(PARBPM) "PAR BPMs"
        set apsSCRSystemDescription(StreakCamera) "Streak Camera"
        set apsSCRSystemDescription(SparkBPMs) "Spark BPMs"
        set apsSCRSystemDescription(SRSCDUBits) "SR BPM SCDU Status Bits"
        set apsSCRSystemDescription(IDBM) "SR ID/BM readbacks"
        set apsSCRSystemDescription(SRXrayBPM) "SR Xray BPM readbacks"
        set apsSCRSystemDescription(SRXrayBPMWaveform) "SR Xray BPM waveforms"
        set apsSCRSystemDescription(BBPMs) "Booster BPM controls"
        set apsSCRSystemDescription(BBPMWaveform) "Booster BSP-100 BPM timing controls"
        set apsSCRSystemDescription(BoosterTune) "Booster Tune measurements"
        set apsSCRSystemDescription(IDEncoder) "ID Encoders"
        set apsSCRSystemDescription(SROrbitSetPt) "SR Orbit Setpoints and Offsets\nNB: This system is usually used for recovery of setpoints after an error in steering."
        set apsSCRSystemDescription(SRCorSetpts) "SR Corrector Setpoints\nNB: This system is usually used for recovery of setpoints after an error in steering."
        set apsSCRSystemDescription(LPL) "Linac, PAR, and LET"
        set apsSCRSystemDescription(LTS) "RFGun Test Stand"
        set apsSCRSystemDescription(SRVideo) "SR video system for S35BM beamline"
        set apsSCRSystemDescription(LI:VD1) "MV200 LI:VD1 video system for all beamlines."
        set apsSCRSystemDescription(DG535) "DG535"
        set apsSCRSystemDescription(Dimtel) "Dimtel systems"
        set apsSCRSystemDescription(RTFBdaq) "DAQ RTFB"
        set apsSCRSystemDescription(TetrAMM) "TetrAMM"
        
        set apsSCRGroupDescription(Special) "Special systems"
        set apsSCRGroupDescription(SR) "SR systems"
        set apsSCRGroupDescription(Booster) "Booster systems"
        set apsSCRGroupDescription(LPL) "LINAC/PAR/LET systems"
        set apsSCRGroupDescription(MV200) "MV200 systems"
        set apsSCRGroupDescription(Obsolete) "Obsolete systems"
        set apsSCRGroupMenuOrder {LPL Booster SR Special MV200 Obsolete}
        
        set apsSCRGroupMembers(Special) {DG535 IDEncoder SROrbitSetPt SRCorSetpts TetrAMM}
        set apsSCRGroupMembers(SR) {SR SRF SBPMs SBPMWaveform SBPMLibera SBPMLiberaWaveform SRVideo XrayTranslation MPS SRNASA SRSCDUBits IDBM SRXrayBPM SRXrayBPMWaveform P0Feedback TFeedback Dimtel RTFBdaq}
        set apsSCRGroupMembers(Booster) {Booster BRF BBPMs BBPMWaveform BoosterTune}
        set apsSCRGroupMembers(LPL) {LPL PAR PARNASA PARBPM LTS SparkBPMs StreakCamera}
        set apsSCRGroupMembers(Obsolete) {LINAC LEUTL PLET}
        set apsSCRGroupMembers(MV200) {LI:VD1}
        set apsSCRAccelManagerAccount(PLET) "borland"
        set apsSCRAccelManagerAccount(SR)   "sr"
        set apsSCRAccelManagerAccount(Booster) "sereno"
        set apsSCRAccelManagerAccount(BRF) "sereno"
        set apsSCRAccelManagerAccount(LINAC) "borland"
        set apsSCRAccelManagerAccount(LEUTL) "leutl"
        set apsSCRAccelManagerAccount(SRF) "sr"
        set apsSCRAccelManagerAccount(SBPMs) "sr"
        set apsSCRAccelManagerAccount(SBPMLibera) "sr"
        set apsSCRAccelManagerAccount(SBPMWaveform) "sr"
        set apsSCRAccelManagerAccount(SBPMLiberaWaveform) "sr"
        set apsSCRAccelManagerAccount(XrayTranslation) "sr"
        set apsSCRAccelManagerAccount(MPS) "sr"
        set apsSCRAccelManagerAccount(SRNASA) "sr"
        set apsSCRAccelManagerAccount(PARNASA) "borland"
	set apsSCRAccelManagerAccount(PARBPM) "par"
	set apsSCRAccelManagerAccount(PAR) "par"
        set apsSCRAccelManagerAccount(StreakCamera) "par"
        set apsSCRAccelManagerAccount(SparkBPMs) "par"
        set apsSCRAccelManagerAccount(SRSCDUBits) "sr"
        set apsSCRAccelManagerAccount(IDBM) "sr"
        set apsSCRAccelManagerAccount(SRXrayBPM) "sr"
        set apsSCRAccelManagerAccount(SRXrayBPMWaveform) "sr"
        set apsSCRAccelManagerAccount(P0Feedback) "sr"
        set apsSCRAccelManagerAccount(TFeedback) "sr"
        set apsSCRAccelManagerAccount(BBPMs) "sereno"
        set apsSCRAccelManagerAccount(BBPMs) "cyao"
        set apsSCRAccelManagerAccount(BBPMWaveform) "cyao"
        set apsSCRAccelManagerAccount(IDEncoder) "mls"
        set apsSCRAccelManagerAccount(SROrbitSetPt) "sr"
        set apsSCRAccelManagerAccount(SRCorSetpts) "sr"
        set apsSCRAccelManagerAccount(SRVideo) "sr"
        set apsSCRAccelManagerAccount(LTS) "lewellen"
        set apsSCRAccelManagerAccount(LPL) "linac"
        set apsSCRAccelManagerAccount(LI:VD1) "par"
        set apsSCRAccelManagerAccount(DG535) "jbs"
        set apsSCRAccelManagerAccount(Dimtel) "sr"
        set apsSCRAccelManagerAccount(RTFBdaq) "sr"
        set apsSCRAccelManagerAccount(TetrAMM) "sr"
    }
        
    set apsSCRSystemList [array names apsSCRSystemDescription]
    set apsSCRSystem [lindex $apsSCRSystemList 0]
    set apsSCRGroup ""

    foreach item $apsSCRSystemList {
        set apsSCRCustomCompareList($item) ""
        set apsSCRCustomCompareDescription($item) ""
    }
    
    foreach group {H V Q S} {
        lappend apsSCRCustomCompareList(SR) \
          "APSSCRSRCompareConvertors -group $group -readback 1" \
          "APSSCRSRCompareConvertors -group $group -readback 0" 
        lappend apsSCRCustomCompareDescription(SR) \
          "$group converter readbacks" \
          "$group converter setpoints" 
    }
    lappend apsSCRCustomCompareList(SR) \
      APSSCRSRCompareOrbitSetpoints "APSSCRSRCompareOrbitSetpoints -mode orbitCorrection" APSSCRSRCompareOrbitOffsets 

    lappend apsSCRCustomCompareDescription(SR) \
      "Orbit setpoints" "Orbit setpoints in orbit correction" "BPM offsets"

    lappend apsSCRCustomCompareList(SROrbitSetPt) \
      APSSCRSRCompareOrbitSetpoints "APSSCRSRCompareOrbitSetpoints -mode orbitCorrection" APSSCRSRCompareOrbitOffsets 

    lappend apsSCRCustomCompareDescription(SROrbitSetPt) \
      "Orbit setpoints" "Orbit setpoints in orbit correction" "BPM offsets"

    lappend apsSCRCustomCompareList(XrayTranslation) \
      "APSSCRXrayTranslationCompare -mode 1only" \
      "APSSCRXrayTranslationCompare -mode 2only" \
      "APSSCRXrayTranslationCompare -mode 1of1w2of2" \
      "APSSCRXrayTranslationCompare -mode 2of1w1of2"
    lappend apsSCRCustomCompareDescription(XrayTranslation) \
      "Controller \#1 only" "Controller \#2 only" \
      "Controller \#1 of filename 1 with Controller \#2 of filename 2" \
      "Controller \#2 of filename 1 with Controller \#1 of filename 2" 
    lappend apsSCRCustomCompareList(SRXrayBPM) \
      "APSSRXrayBPMCustomCompare -bpmType ID -plane x -gapcompare 0" \
      "APSSRXrayBPMCustomCompare -bpmType ID -plane y -gapcompare 0" \
      "APSSRXrayBPMCustomCompare -bpmType BM -plane y" \
      "APSSRXrayBPMCustomCompare -bpmType ID -gapcompare 1" \
      "APSSRXrayBPMCustomCompare -bpmType ID -bladescompare 1 -group a -type Rate" \
      "APSSRXrayBPMCustomCompare -bpmType ID -bladescompare 1 -group a -type Offset" \
      "APSSRXrayBPMCustomCompare -bpmType ID -bladescompare 1 -group b -type Rate" \
      "APSSRXrayBPMCustomCompare -bpmType ID -bladescompare 1 -group b -type Offset" 
    lappend apsSCRCustomCompareDescription(SRXrayBPM) \
      "ID x readbacks" \
      "ID y readbacks" \
      "BM y readbacks" \
      "ID gap readbacks" \
      "ID defaultA blade rates" \
      "ID defaultA blade offsets" \
      "ID defaultB blade rates" \
      "ID defaultB blade offsets"

    lappend apsSCRCustomReviewList(SR) \
      "APSSRPlotSCRBPMData -field SetpointAO"  \
      "APSSRPlotSCRBPMData -field OffsetAO" 
    lappend apsSCRCustomReviewDescription(SR) \
      "BPM Setpoints" \
      "BPM Offsets" 

    lappend apsSCRCustomReviewList(SBPMs) \
      "APSSRPlotSCRBPMData -field GainAO" 
    lappend apsSCRCustomReviewDescription(SBPMs) \
      "BPM Gains"

    # check "system" subdirectories 
    set notInList ""
    foreach filename [lsort [glob -nocomplain $apsSCRSnapDir/*]] {
        if {[file isdirectory $filename] && \
              [lsearch -exact $apsSCRSystemList [file tail $filename]]==-1} {
            lappend notInList [file tail $filename]
        }
    }
    foreach file $apsSCRSystemList {
        set file0 $apsSCRSnapDir/$file
        if {![file exists $file0] || ![file isdirectory $file0]} {
            return -code error "System directory for $file not found."
        }
    }
    if [llength $notInList] {
        puts stderr "Warning: directories for these systems exist but are not accessed: $notInList"
        return -code ok "Warning: directories for these systems exist but are not accessed: $notInList"
    }
}

if {(![info exists apsSkipSCRDefineVariables]) || ($apsSkipSCRDefineVariables == 0)} {
    if {[catch {APSSCRDefineVariables} apsSCRDefineVariablesResults]} {
        global tk_version
        if {[info exists tk_version]} {
            update
            APSAlertBox .errorMessage -errorMessage "Error (APSSCRDefineVariables): $apsSCRDefineVariablesResults"
            exit
        } else {
            puts stderr "Error (APSSCRDefineVariables): $apsSCRDefineVariablesResults"
            exit
        }
    }
}

proc APSSCRSRCompareConvertors {args} {
    set machine ""
    set snapshot1 ""
    set snapshot2 ""
    set readback 0
    set group H
    set plots 1
    set fileDisplay 1
    APSStrictParseArguments {machine snapshot1 snapshot2 readback group plots fileDisplay} 
    global SCRdebug apsSCRTopDirectory
    if $SCRdebug { puts stderr "APSSCRSRCompareConvertors: snapshots: $snapshot1 $snapshot2"}

    if {![string length $snapshot1] || ![string length $snapshot2] || \
          ![string length $machine]} {
        return -code error "APSSCRSRCompareConvertors: bad arguments\nmachine=$machine  snapshot1=$snapshot1  snapshot2=$snapshot2"
    }

    set groupTagList {H V Q S}
    set groupNameList {"H corrector" "V corrector" "Quadrupole" "Sextupole"}
    set groupMatchStringList {S[1-9]*:H[0-4]: S[1-9]*:V[0-4]: S[1-9]*:Q[1-5]: S[1-9]*:S[1-4]:}
    set groupPVCountList {323 323 400 280}
    set groupIndex [lsearch $groupTagList $group]
    if $groupIndex==-1 {
        return -code error "APSSCRSRCompareConvertors: group $group is unknown."
    }
    set groupName [lindex $groupNameList $groupIndex]
    set pvCount [lindex $groupPVCountList $groupIndex]
    set matchString [lindex $groupMatchStringList $groupIndex]

    if $readback {
        set title "$groupName readback changes from [file tail [file rootname $snapshot1]] to [file tail [file rootname $snapshot2]]"
        set currentType CurrentAI
    } else {
        set title "$groupName setpoint changes from [file tail [file rootname $snapshot1]] to [file tail [file rootname $snapshot2]]"
        set currentType CurrentAO
    }

    set tmpFile /tmp/[APSTmpString]
    set tmpID [APSUniqueName tmpID]
    APSAddToTmpFileList -ID $tmpID -fileList [list $tmpFile $tmpFile.print]

    if [catch {exec sddscombine $snapshot1 $snapshot2 -pipe=out \
                 | sddsprocess -pipe -match=column,ControlName=$matchString$currentType \
                 -scan=column,${currentType},ValueString,%le,units=A \
                 -edit=column,ElementName,ControlName,%/:${currentType}// \
                 | sddsxref $apsSCRTopDirectory/miscInputs/ConverterMagnetPositions.xref \
                 -pipe -match=ElementName -take=s,Sector -reuse=page,row -nowarning \
                 | sddsselect -pipe $snapshot1 -match=ControlName -reuse=page -nowarning \
                 | sddsselect -pipe $snapshot2 -match=ControlName -reuse=page -nowarning \
                 | sddssort -pipe -column=s,incr \
                 | sddschanges -pipe -copy=s,ElementName -changesIn=${currentType} \
                 | sddsprocess -pipe \
                 "-redefine=column,Sector,s 27.6 / 1 + Sector -,type=double" \
                 | tee $tmpFile \
                 | sddsprintout -pipe=in $tmpFile.print \
                 -column=ElementName -column=ChangeIn${currentType} "-title=$title"} result] {
        APSDeleteTmpFileList -ID $tmpID
        return -code error "$result"
    }

    if $fileDisplay {
        APSFileDisplayWindow [APSUniqueName .] \
          -comment "$title" -fileName $tmpFile.print -deleteOnClose 1 -width 80 \
          -contextHelp "$title" -sddsExportableFile $tmpFile 
        update
    }

    if $plots {
        exec sddsplot "-topline=$title" \
          $tmpFile -split=page -sep=page -title=  \
          -column=Sector,ChangeIn${currentType} -graph=impulse,type=1 \
          -legend=spec=>15A \
          -column=Sector,ChangeIn${currentType} -graph=impulse,type=2 \
          -filter=column,ChangeIn${currentType},-15,15 -legend=spec=>1.5A \
          -column=Sector,ChangeIn${currentType} -graph=impulse,type=0 \
          -filter=column,ChangeIn${currentType},-1.5,1.5 -legend=spec=<1.5A &
    }
}

proc APSSCRSRCompareOrbitSetpoints {args} {
    return [eval APSSCRSRCompareBPMQuantities -quantity SetpointAO $args]
}
proc APSSCRSRCompareOrbitOffsets {args} {
    return [eval APSSCRSRCompareBPMQuantities -quantity OffsetAO $args]
}

proc APSSCRSRCompareBPMQuantities {args} {
    set quantity SetpointAO
    set machine ""
    set snapshot1 ""
    set snapshot2 ""
    set plots 1
    set fileDisplay 1
    set mode ""
    APSStrictParseArguments {machine snapshot1 snapshot2 plots fileDisplay quantity mode} 
    global SCRdebug apsSCRTopDirectory

    if {![string length $snapshot1] || ![string length $snapshot2] || \
          ![string length $machine]} {
        return -code error "APSSCRSRCompareBPMQuantities: bad arguments"
    }
    if [file mtime $snapshot1]>[file mtime $snapshot2] {
        set snapshotTemp $snapshot1
        set snapshot1 $snapshot2
        set snapshot2 $snapshotTemp
    }
    
    set title "$quantity changes from [file tail [file rootname $snapshot1]] to [file tail [file rootname $snapshot2]]"

    set tmpFile /tmp/[APSTmpString]
    set tmpID [APSUniqueName tmpID]
    APSAddToTmpFileList -ID $tmpID -fileList \
      [list $tmpFile $tmpFile.x $tmpFile.y $tmpFile.print \
         $tmpFile.ss2 $tmpFile.ss1]
    APSAddToTmpFileList -ID semi-permanent -fileList $tmpFile.xy.exportable

    if [catch {exec sddsprocess $snapshot1 $tmpFile.ss1 \
                 -edit=col,BPMName,ControlName,%/:ms://%/x://%/y://%/${quantity}// \
                 -match=column,ControlName=S*:P?:ms:?:${quantity} \
             } result] {
        APSDeleteTmpFileList -ID $tmpID
        return -code error "APSSCRSRCompareBPMQuantities(0): $result"
    }

    # with orbit correction mode, we have to separate the x and y planes
    # into different files, then do the comparison.
    if [string match $mode orbitCorrection] {
        # create file of selected bpms in x and y planes and join them.
        if [catch {exec sddsprocess /home/helios/oagData/sr/orbitControllaw/lattices/default/h.default/config $tmpFile.xbpms \
                     -match=para,NameType=MonitorNames \
                     -filter=col,Flag,1,1 \
                 } result] {
            APSDeleteTmpFileList -ID $tmpID
            return -code error "APSSCRSRCompareBPMQuantities(1): $result"
        }
        
        if [catch {exec sddsprocess /home/helios/oagData/sr/orbitControllaw/lattices/default/v.default/config $tmpFile.ybpms \
                     -match=para,NameType=MonitorNames \
                     -filter=col,Flag,1,1\
                 } result] {
            APSDeleteTmpFileList -ID $tmpID
            return -code error "APSSCRSRCompareBPMQuantities(2): $result"
        }

        foreach coord {x y} {
            # $tmpFile.ss1.${coord}.oc is a smaller snapshot file.
            if [catch {exec sddsselect $tmpFile.ss1 $tmpFile.${coord}bpms -pipe=out \
                         -match=BPMName=Name -reuse \
                         | sddsprocess -pipe=in $tmpFile.ss1.${coord}.oc \
                         -match=col,ControlName=*${coord}* \
                     } result] {
                APSDeleteTmpFileList -ID $tmpID
                return -code error "APSSCRSRCompareBPMQuantities(3): $result"
            }
            # take intersection of snapshot1 and snapshot2 to make smaller snapshot
            # file $tmpFile.ss2.${coord}.oc.
            if [catch {exec sddsselect $snapshot2 $tmpFile.ss1.${coord}.oc -pipe=out \
                         -match=ControlName -reuse \
                         | sddsprocess -pipe=in $tmpFile.ss2.${coord}.oc \
                         -edit=col,BPMName,ControlName,%/:ms://%/x://%/y://%/${quantity}// \
                         -match=col,ControlName=*${coord}* \
                     } result] {
                APSDeleteTmpFileList -ID $tmpID
                return -code error "APSSCRSRCompareBPMQuantities(4): $result"
            }
        }
        foreach coord {x y} {
            if [catch {exec sddscombine $tmpFile.ss1.${coord}.oc $tmpFile.ss2.${coord}.oc -pipe=out \
                         | sddsprocess -pipe \
                         -print=para,Plane,${coord} \
                         -scan=column,${quantity}Value,ValueString,%le,units=mm \
                         | sddssort -pipe -column=ControlName -unique \
                         | sddsxref $apsSCRTopDirectory/miscInputs/BPMPositions.xref \
                         -pipe -match=BPMName -take=s -reuse=page,row -nowarning \
                         | sddssort -pipe -column=s,incr \
                         | sddschanges -pipe \
                         -copy=s,BPMName -changesIn=${quantity}Value \
                         | sddsprocess -pipe=in $tmpFile.${coord} \
                         "-define=column,Sector,s 27.6 / 1 +" \
                     } result] {
                APSDeleteTmpFileList -ID $tmpID
                return -code error "APSSCRSRCompareBPMQuantities(5): $result"
            }
        }
        if [catch {exec sddsconvert $tmpFile.x -pipe=out \
                     -rename=column,ChangeIn${quantity}Value=x${quantity}Delta \
                     | sddsprintout -pipe \
                     -para=Plane -column=BPMName \
                     -column=?${quantity}Delta "-title=$title" > $tmpFile.print 
            exec sddsconvert $tmpFile.y -pipe=out \
                     -rename=column,ChangeIn${quantity}Value=y${quantity}Delta \
                     | sddsprintout -pipe \
                     -para=Plane -column=BPMName \
                     -column=?${quantity}Delta "-title=$title" >> $tmpFile.print \
                 } result] {
            APSDeleteTmpFileList -ID $tmpID
            return -code error "APSSCRSRCompareBPMQuantities(6): $result"
        }

    } else {
        # take intersection of sets from snapshot1 and snapshot2
        if [catch {exec sddsselect $snapshot2 $tmpFile.ss1 -pipe=out -match=ControlName \
                     | sddsprocess -pipe=in $tmpFile.ss2 \
                     -edit=col,BPMName,ControlName,%/:ms://%/x://%/y://%/${quantity}// \
                 } result] {
            APSDeleteTmpFileList -ID $tmpID
            return -code error "APSSCRSRCompareBPMQuantities(7): $result"
        }
        
        if [catch {exec sddscombine $tmpFile.ss1 $tmpFile.ss2 -pipe=out \
                     | sddsprocess -pipe \
                     -scan=column,${quantity}Value,ValueString,%le,units=mm \
                     -edit=column,Plane,ControlName,3Z:1fD \
                     | sddssort -pipe -column=ControlName -unique \
                     | sddsxref $apsSCRTopDirectory/miscInputs/BPMPositions.xref \
                     -pipe -match=BPMName -take=s -reuse=page,row -nowarning \
                     | sddssort -pipe -column=Plane,incr -column=s,incr \
                     | sddschanges -pipe=in $tmpFile \
                     -copy=s,Plane,BPMName -changesIn=${quantity}Value \
                 } result] {
            APSDeleteTmpFileList -ID $tmpID
            return -code error "APSSCRSRCompareBPMQuantities(8): $result"
        }
        
        if [catch {exec sddsprocess $tmpFile $tmpFile.x -match=column,Plane=x  \
                     "-define=column,Sector,s 27.6 / 1 +"\
                 } result] {
            APSDeleteTmpFileList -ID $tmpID
            return -code error "APSSCRSRCompareBPMQuantities(9): $result"
        }
        if [catch {exec sddsprocess $tmpFile $tmpFile.y -match=column,Plane=y  \
                     "-define=column,Sector,s 27.6 / 1 +"\
                 } result] {
            APSDeleteTmpFileList -ID $tmpID
            return -code error "APSSCRSRCompareBPMQuantities(10): $result"
        }
        
        if [catch {exec sddsconvert $tmpFile.x -pipe=out \
                     -rename=column,ChangeIn${quantity}Value=x${quantity}Delta \
                     | sddsxref $tmpFile.y -pipe -take=* \
                     | sddsconvert -pipe -rename=column,ChangeIn${quantity}Value=y${quantity}Delta \
                     | tee $tmpFile.xy.exportable \
                     | sddsprintout -pipe=in $tmpFile.print \
                     -column=BPMName -column=?${quantity}Delta "-title=$title"\
                 } result] {
            APSDeleteTmpFileList -ID $tmpID
            return -code error "APSSCRSRCompareBPMQuantities(11): $result"
        }
    }

    if $fileDisplay {
        APSFileDisplayWindow [APSUniqueName .] \
          -comment "$title" -fileName $tmpFile.print -deleteOnClose 1 -width 80 \
          -contextHelp "$title" -sddsExportableFile $tmpFile.xy.exportable 
        update
    }

    if $plots {
        exec sddsplot -split=page -sep=page -title= -limit=ymax=0.5,ymin=-0.5,autoscaling \
          -column=Sector,ChangeIn${quantity}Value "-topline=x $title" \
          -graph=impulse,type=0 $tmpFile.x -legend=spec=Other \
          -column=Sector,ChangeIn${quantity}Value $tmpFile.x \
          -graph=impulse,type=1 -legend=spec=P1 -match=col,BPMName=*P1* \
          -column=Sector,ChangeIn${quantity}Value $tmpFile.x \
          -graph=impulse,type=2 -legend=spec=P3+P4 -match=col,BPMName=*P\[34\]* \
          -end \
          -column=Sector,ChangeIn${quantity}Value "-topline=y $title" \
          -graph=impulse,type=0  $tmpFile.y -legend=spec=Other \
          -column=Sector,ChangeIn${quantity}Value $tmpFile.y \
          -graph=impulse,type=1 -legend=spec=P1 -match=col,BPMName=*P1* \
          -column=Sector,ChangeIn${quantity}Value $tmpFile.y \
          -graph=impulse,type=2 -legend=spec=P3+P4 -match=col,BPMName=*P\[34\]* &
    }
    after 60000 "APSDeleteTmpFileList -ID $tmpID"
}

proc APSSCRXrayTranslationCompare {args} {
    set machine ""
    set snapshot1 ""
    set snapshot2 ""
    set mode ""
    APSStrictParseArguments {machine snapshot1 snapshot2 mode} 
    if {![string length $snapshot1] || ![string length $snapshot2] || \
          ![string length $machine] || ![string length $mode] } {
        return -code error "APSSCRXrayTranslationCompare: bad arguments"
    }
    set tmpfile /tmp/[APSTmpString]
    set tmpID [APSUniqueName tmpID]
    APSAddToTmpFileList -ID $tmpID -fileList \
      [list $tmpfile.snap1 $tmpfile.snap2 $tmpfile.change $tmpfile.print]
    APSAddToTmpFileList -ID semi-permanent -fileList $tmpfile.exportable
    switch $mode {
             1only {
            set controller(snap1) 1
            set controller(snap2) 1
        }
        2only {
            set controller(snap1) 2
            set controller(snap2) 2
        }
        1of1w2of2 {
            set controller(snap1) 1
            set controller(snap2) 2
        }
        2of1w1of2 {
            set controller(snap1) 2
            set controller(snap2) 1
        }
    }
    set title "Comparison of controller \#$controller(snap1) in [file tail $snapshot1] and controller \#$controller(snap2) in [file tail $snapshot2]"
    if [catch {exec sddsprocess $snapshot1 $tmpfile.snap1 \
                 -match=col,ControlName=SXDRV:$controller(snap1):* \
                 -edit=col,Rootname,ControlName,%/SXDRV:$controller(snap1)://
        exec sddsprocess $snapshot2 $tmpfile.snap2 \
                 -match=col,ControlName=SXDRV:$controller(snap2):* \
                 -edit=col,Rootname,ControlName,%/SXDRV:$controller(snap2)://
        
        exec sddscombine $tmpfile.snap1 $tmpfile.snap2 -pipe=out \
                 | sddsprocess -pipe \
                 -scan=column,Value,ValueString,%le,units=mm \
                 | sddssort -pipe -column=Rootname -unique \
                 | sddschanges -pipe=in $tmpfile.changes \
                 -copy=Rootname -changesIn=Value
        
        exec sddsconvert $tmpfile.changes -pipe=out \
                 -delete=col,ControlName \
                 -rename=column,ChangeInValue=Delta \
                 | sddsxref -pipe $tmpfile.snap1 \
                 -match=Rootname -take=ControlName,ValueString \
                 | sddsconvert -pipe \
                 -rename=col,ControlName=ControlNameSnap1 \
                 -rename=col,ValueString=ValueStringSnap1 \
                 | sddsxref -pipe $tmpfile.snap2 \
                 -match=Rootname -take=ControlName,ValueString \
                 | sddsconvert -pipe \
                 -rename=col,ControlName=ControlNameSnap2 \
                 -rename=col,ValueString=ValueStringSnap2 \
                 | sddsprocess -pipe \
                 -scan=col,ValueSnap1,ValueStringSnap1,%lf \
                 -scan=col,ValueSnap2,ValueStringSnap2,%lf \
                 | sddsprintout -pipe=in $tmpfile.print \
                  -formatDefaults=double=%12.6f \
                 -column=Rootname,format=%22s \
                 -column=(ValueSnap1,ValueSnap2,Delta) \
                 "-title=$title" \
             } result] {
        APSDeleteTmpFileList -ID $tmpID
        return -code error "$result"
    }
    
    APSFileDisplayWindow [APSUniqueName .] \
      -comment "$title" -fileName $tmpfile.print -deleteOnClose 1 -width 80 \
      -contextHelp "$title" -sddsExportableFile $tmpfile.exportable 
    return
}

#this procedures sets SR:TUNE:Mux4Sw2BO to NASA or VSA and
#SR:TUNE:Mux4MO in both mode
proc APSSetTuneMultiplexer {args} {
    set mode ""
    set sum 0
    APSParseArguments {mode sum} 

    set mode [string toupper $mode]
    switch $mode {
        VSA {
            set drive VSA
        }
        NASA {
            set drive NASA
        }
        CAVITY {
            #set drive Cavity/VSA
            #set mode Cavity
	    set drive VSA
	    set mode1 "Cavity #1"
        }
        SUM {
            #set drive Cavity/VSA
            #set mode "S2 SUM"
	    set drive VSA
	    set mode1 "S4 Sum"
        }
        default {
            return -code error "Invalid mode given, should be either VSA or NASA"
        }
    }
    if [catch {exec cavget -list=SR:TUNE:Mux4Sw0BO -pend=10} origDrive] {
	return -code error "APSSetTuneMultiplexer0: Unable to read SR:TUNE:Mux4Sw0BO: $origDrive"
    }
    if [string compare $origDrive $drive] {
	if [catch {exec cavput -list=SR:TUNE:Mux4Sw0BO=$drive -pend=10} result] {
	    return -code error "APSSetTuneMultiplexer: Unable to set SR:TUNE:Mux4Sw0BO: $result"
	}
    }
    switch $mode {
	NASA -
	VSA {
	    if [catch {exec cavget -list=SR:TUNE:HP_1366A_C0P0_MBBO -pend=10} origMode] {
		return -code error "APSSetTuneMultiplexer(2): Unable to read  SR:TUNE:HP_1366A_C0P0_MBBO: $origMode"
	    }
	    if [string compare $mode $origMode] {
		if [catch {exec cavput -list=SR:TUNE:HP_1366A_C0P0_MBBO=$mode -pend=10} result] {
		    return -code error "APSSetTuneMultiplexer(2a): Unable to set  SR:TUNE:HP_1366A_C0P0_MBBO: $result"
		}
	    }
	}
	CAVITY -
	SUM {
	    #drive should be off for cavity/sum signal
	    if [catch {exec cavget -list=SR:TUNE:HP_1366A_C0P1_MBBO -pend=30} origMode] {
		return -code error "APSSetTuneMultiplexer(3): $origMode"
	    }
	    if [string compare $origMode "OFF"]!=0 {
		if [catch {exec cavput -list=SR:TUNE:HP_1366A_C0P1_MBBO=OFF -pend=30} result] {
		    return -code error "APSSetTuneMultiplexer(4): unable to set SR:TUNE:HP_1366A_C0P1_MBBO to $mode: $result"
		}
	    }
	    #cavity sum mode
	    if [catch {exec cavput "-list=SR:TUNE:HP_1366A_C2P1_MBBO=$mode1" } result] {
		return -code error "APSSetTuneMultiplexer(4): unable to set SR:TUNE:HP_1366A_C2P1_MBBO to $mode1: $result"
	    }
	}
    }
}

proc APSSCRProcessXrayBPMSnapshot {args} {
    set inputFile ""
    set outputFile ""
    set bpmType ""
    set plane ""
    APSStrictParseArguments {inputFile outputFile bpmType plane} 
    if {![string length $inputFile]} {
        return -code error "APSSCRProcessXrayBPMSnapshot: input file is not provided!"
    }
    if {![string length $outputFile]} {
        return -code error "APSSCRProcessXrayBPMSnapshot: output filename is not provided!"
    }
    if ![string length $bpmType] {
        return -code error "APSSCRProcessXrayBPMSnapshot: Xray bpm type (ID or BM) is not provided!"
    }
    if ![string length $plane] {
        return -code error "APSSCRProcessXrayBPMSnapshot: Xray bpm plane (x or y) is not provided!"
    }
    set bpmType [string toupper $bpmType]
    set plane [string tolower $plane]
    if {$bpmType!="ID" && $bpmType!="BM"} {
        return -code error "APSSCRProcessXrayBPMSnapshot: invalid Xray bpm type, shoule be ID or BM"
    }
    if {$plane!= "x" && $plane !="y"} {
        return -code error "APSSCRProcessXrayBPMSnapshot: invalid Xray bpm plane, should be x or y"
    }
    if {$plane=="x" && $bpmType=="BM"} {
        return -code error "APSSCRProcessXrayBPMSnapshot: invalid plane input for BM xray, should be y only"
    }
    set tmpRoot /tmp/[APSTmpString]
    set fileList ""
    lappend fileList $tmpRoot
    if [catch {exec sddsprocess  $inputFile \
                 -match=col,IsNumerical=y \
                 -scan=col,Value,ValueString,%le -pipe=out \
                 | sddsconvert -pipe=in -retain=col,ControlName,Value $tmpRoot 
        exec sddsprocess $tmpRoot -pipe=out -match=col,ControlName=*${bpmType}* \
                 | sddstranspose -pipe=in ${tmpRoot}.$bpmType } result] {
        return -code error "APSSCRProcessXrayBPMSnapshot: $result"
    }
    lappend fileList ${tmpRoot}.$bpmType
    if [catch {exec sddscollect ${tmpRoot}.$bpmType -pipe=out \
                 -collect=suffix=:P1:msAve:${plane},column=${plane}bpm1 \
                 -collect=suffix=:P2:msAve:${plane},column=${plane}bpm2 \
                 | sddsprocess -pipe -scan=col,Sector,Rootname,S%ld,type=long \
                 | sddssort -pipe -col=Sector \
                 | sddsconvert -pipe=in -delete=col,Rootname ${tmpRoot}.bpm } result] {
        return -code error "APSSCRProcessXrayBPMSnapshot: $result"
    }
    lappend fileList ${tmpRoot}.bpm
    if {$bpmType=="ID"} {
        if [catch {exec sddscollect ${tmpRoot}.$bpmType -pipe=out \
                       -collect=suffix=:Gap,column=gap \
                       | sddsprocess -pipe=in ${tmpRoot}.gap \
                       -scan=col,Sector,Rootname,ID%2d,type=long
            exec sddsprocess ${tmpRoot}.gap -match=col,Rootname=*us* -pipe=out -nowarnings \
                       | sddsconvert -pipe=in ${tmpRoot}.us -rename=col,gap=USgap
            exec sddsprocess ${tmpRoot}.gap "-match=col,Rootname=*us*,!" -pipe=out \
                       -reedit=col,Rootname,%/ds// \
                       | sddsxref -pipe=in ${tmpRoot}.us ${tmpRoot}.gap1a -equate=Sector \
                       -fillin -take=USgap -nowarnings 
            exec sddsselect $tmpRoot.us $tmpRoot.gap1a -invert -equate=Sector -pipe=out \
                       | sddsprocess -pipe=in $tmpRoot.gap2 -define=col,gap,0.0 -nowarnings
            exec sddscombine $tmpRoot.gap1a $tmpRoot.gap2 -merge -pipe=out \
                       | sddssort -pipe=in -col=Rootname $tmpRoot.gap1 \
                   } result] {
            return -code error "APSSCRProcessXrayBPMSnapshot: $result"
        }
        lappend fileList "${tmpRoot}.gap ${tmpRoot}.us ${tmpRoot}.gap1 $tmpRoot.gap1a $tmpRoot.gap2"
    }
    foreach set {1 2} {
        if [catch {exec sddsconvert ${tmpRoot}.$bpmType -retain=col,*:P${set}:*:AI -pipe=out \
                     | sddscollect -pipe -collect=suffix=:AI,column=blade${set} \
                     | sddsprocess -pipe -scan=col,Sector,Rootname,S%ld,type=long \
                     | sddssort -pipe -col=Sector \
                     | sddsbreak -pipe -changeof=Sector \
                     | sddsprocess -pipe -process=blade${set},max,maxV${set} \
                     -process=Sector,first,Sector \
                     | sddsconvert -pipe -retain=par,Sector,max* \
                     | sddscollapse -pipe \
                     | sddscast -pipe=in ${tmpRoot}.blade${set} \
                     -cast=col,Sector,double,long } result] {
            return -code error "APSSCRProcessXrayBPMSnapshot: $result"
        }
        lappend fileList ${tmpRoot}.blade${set}
    }
    if {$bpmType=="ID"} {
        if [catch {exec sddsxref ${tmpRoot}.bpm ${tmpRoot}.gap1 ${tmpRoot}.bpmgap \
                     -equate=Sector -take=Rootname,gap,USgap -nowarnings } result] {
            return -code error "APSSCRProcessXrayBPMSnapshot: $result"
        }
        lappend fileList ${tmpRoot}.bpmgap
        set firstFile ${tmpRoot}.bpmgap
    } else {
        set firstFile ${tmpRoot}.bpm
    }
    if [catch {eval exec sddsxref ${tmpRoot}.blade1 ${tmpRoot}.blade2 ${tmpRoot}.blade \
                 -equate=Sector -take=maxV2 -nowarnings
        eval exec sddsxref $firstFile ${tmpRoot}.blade -pipe=out -equate=Sector \
                 -take=maxV1,maxV2 \
                 -nowarnings \
                 | sddsprocess -pipe \
                 -reprint=col,Rootname,${bpmType}%.2d,Sector \
                 | sddssort -pipe=in -col=Rootname $outputFile } result] {
        return -code error "APSSCRProcessXrayBPMSnapshot: $result"
    }
    lappend fileList ${tmpRoot}.blade
    APSAddToTmpFileList -ID processXraySnapshot -fileList $fileList
}

proc APSSRXrayBPMCustomCompare {args} {
    global apsSCRRequestDir
    set machine ""
    set snapshot1 ""
    set snapshot2 ""
    set snap1Label ""
    set snap2Label ""
    set Value1Label ""
    set Value2Label ""
    set DifferenceLabel ""
    set posDiffTolerance 0.1
    set gapDiffTolerance 0.1
    #minVoltage is the minimum acceptable blade voltage
    set minVoltage 0.01
    set bpmType ""
    set plane x
    set gapcompare 0
    set bladescompare 0
    set type Rate
    set group a
    if {[APSStrictParseArguments {machine snapshot1 snapshot2 snap1Label snap2Label DifferenceLabel \
                                    posDiffTolerance  gapDiffTolerance minVoltage \
                                    bpmType plane gapcompare \
                                    Value1Label Value2Label bladescompare type group\
                                } ]==-1} {
        return -code error "Bad arguments to APSSRXrayBPMCustomCompare"
    } 
    if {![string length $bpmType] || ![string length $plane] } {
        return -code error "APSSRXrayBPMCustomCompare: bpmType or plane is not given"
    }
    if {![file exist $snapshot1] || ![file exist $snapshot2] } {
        return -code error "APSSRXrayBPMCustomCompare: snapshot files do not exist!"
    }
    if ![string length $Value1Label] {set Value1Label dataset1}
    if ![string length $Value2Label] {set Value2Label dataset2}
    if ![string length $DifferenceLabel] {set DifferenceLabel data1-data2}
    if ![string length $snap1Label] {set snap1Label $snapshot1}
    if ![string length $snap2Label] {set snap2Label $snapshot2}
    if $gapcompare {
        set tmpfile /tmp/[APSTmpString].snap1
        if [catch {exec sddsprocess $snapshot1 $tmpfile \
                     "-match=col,ControlName=*ID*:Gap" } result] {
            return -code error "APSSRXrayBPMCustomCompare: Snapshot1 does not have ID gap readback bpms!"
        }
        if [catch {APSCompareMachine -machine SRXrayBPM -snapshot1 $tmpfile \
                     -snap1Label $snapshot1 \
                     -snapshot2 $snapshot2 -graphics 1} result] {
            return -code error "Error doing ID gap readbacks compare: $result"
        }
    } elseif $bladescompare  {
        set option ""
        switch $group {
            a {
                set option "-filter=col,DefaultA,0.5,1.5"
            }
            b {
                set option "-filter=col,DefaultB,0.5,1.5"
            }
        }
        set tmpRoot /tmp/[APSTmpString]
        if [catch {exec sddsprocess /home/helios/oagData/sr/IDs/devices.sdds -pipe=out $option \
                 | sdds2stream -pipe -col=DeviceName } IDList] {
            return -code error "error in getting devicenames: $IDList"
        }
        set option ""
        foreach id $IDList {
            set sector [scan $id ID%ld]
            if ![string length $option] {
                append option  -match=col,ControlName=S${sector}ID*${type}P
            } else {
                append option ,ControlName=S${sector}ID*${type}P,|
            }
        }
        if [catch {exec sddsprocess $snapshot1 $tmpRoot.1 "$option"
            exec sddsprocess $snapshot2 $tmpRoot.2 "$option"  } result] {
            return -code error "APSSRXrayBPMCustomCompare: Snapshots does not have ID blade rate or offsets: $result!"
        }
        if [catch {APSCompareMachine -machine SRXrayBPM -snapshot1 $tmpRoot.1 \
                     -snap1Label $snapshot1 \
                     -snapshot2 $tmpRoot.2  -graphics 1} result] {
            return -code error "Error doing ID gap readbacks compare: $result"
        }

    } else {
        set tmpRoot /tmp/[APSTmpString]
        set proc1 ${tmpRoot}.proc1
        set proc2 ${tmpRoot}.proc2
        APSAddToTmpFileList -ID compareXrayBPM -fileList "$proc1 $proc2"
        if [catch {APSSCRProcessXrayBPMSnapshot -inputFile $snapshot1 -outputFile $proc1 -bpmType $bpmType \
                     -plane $plane} result] {
            return -code error "APSSRXrayBPMCustomCompare: $result"
        }
        if [catch {APSSCRProcessXrayBPMSnapshot -inputFile $snapshot2 -outputFile $proc2 -bpmType $bpmType \
                     -plane $plane} result] {
            return -code error "APSSRXrayBPMCustomCompare: $result"
        }
        APSAddToTmpFileList -ID compareXrayBPM -fileList ${tmpRoot}.new
        if [catch {exec sddsconvert $proc2 ${tmpRoot}.new  \
                     -edit=col,*bpm*,ei/New/ \
                     -edit=col,gap,ei/New/ \
                     -edit=col,USgap,ei/New/ \
                     -edit=col,max*,ei/New/ \
                 } result ] {
            return -code error "APSSRXrayBPMCustomCompare: $result"
        }
        set resultsFile ${tmpRoot}.result
        APSAddToTmpFileList -ID compareXrayBPM -fileList "$resultsFile $resultsFile.help"
        if [catch {exec sddsxref $proc1 ${tmpRoot}.new -pipe=out -nowarning \
                     -match=Rootname -take=* \
                     | sddsprocess -pipe  \
                     "-def=col,%sDiff,%sNew %s -,select=*New,edit=%/New//,units=mm" \
                     | sddsprocess -pipe=in  $resultsFile \
                     "-def=col,%sAbs,%s abs,select=*Diff,units=mm" \
                 } result ] {
            return -code error "APSSRXrayBPMCustomCompare: $result"
        }
        if ![string compare $bpmType "ID"] {
            if [catch {exec sddsprocess $resultsFile $resultsFile.help -nowarning \
                         -filter=col,gapDiffAbs,0,$gapDiffTolerance \
                         -filter=col,USgapDiffAbs,0,$gapDiffTolerance \
                         "-def=col,minmaxV1,maxV1New maxV1 min2,units=V" \
                         "-def=col,minmaxV2,maxV2New maxV2 min2,units=V"} result] {
                return -code error "APSSRXrayBPMCustomCompare: $result"
            }
            set rows [exec sdds2stream $resultsFile.help -rows=bare]
            #	puts $rows
            if !$rows {
                return -code error "APSSRXrayBPMCustomCompare: all gap differences are bigger than $gapDiffTolerance (tolerance)"
            }              
        } else {
            APSAddToTmpFileList -ID compareXrayBPM -fileList "$resultsFile $resultsFile.help"
            if [catch {exec sddsprocess $resultsFile $resultsFile.help -nowarning \
                         "-def=col,minmaxV1,maxV1New maxV1 min2,units=V" \
                         "-def=col,minmaxV2,maxV2New maxV2 min2,units=V" \
                     } result ] {
                return -code error "CompareData: $result"
            }
        }
        if [catch {eval APSSCRPrintXrayBPMCompTable -resultsFile $resultsFile.help \
                     -snap1Label \"$snap1Label\" -snap2Label \"$snap2Label\" -DifferenceLabel \"$DifferenceLabel\" \
                     -Value1Label \"$Value1Label\" -Value2Label \"$Value2Label\" \
                     -posDiffTolerance $posDiffTolerance -minVoltage $minVoltage \
                     -bpmType $bpmType -plane $plane} status] {
            return -code error "APSSRXrayBPMCustomCompare: $status"
        }
        return $status
    }
    
}

proc APSSCRPrintXrayBPMCompTable {args} {
    set posDiffTolerance 0.1
    #minVoltage is the minimum acceptable blade voltage
    set minVoltage 0.01
    set resultsFile ""
    set DifferenceLabel ""
    set snap2Label ""
    set snap1Label ""
    set Value1Label ""
    set Value2Label ""
    set gapcompare 0
    set bpmType ""
    set plane ""
    if {[APSStrictParseArguments { resultsFile posDiffTolerance minVoltage \
                                     gapcompare snap1Label snap2Label Value1Label Value2Label \
                                     DifferenceLabel bpmType plane} ]==-1} {
        return -code error "Bad arguments to APSSCRPrintXrayBPMCompTable"
    }
    set good ""
    set questionable  "lowV"
    set mergeFiles ""
    set tmpRoot /tmp/[APSTmpString]
    set tmpfileList ""
    if [catch {exec sddsprocess $resultsFile ${tmpRoot}.print1 \
                 -filter=column,minmaxV1,0,$minVoltage,! \
                 -filter=column,minmaxV2,0,$minVoltage,! \
                 "-print=parameter,Title,Comparison of\n$snap1Label\n to \n$snap2Label\n \
               (minimal acceptable voltage $minVoltage V \n minimal acceptable position difference $posDiffTolerance mm)" \
                 -print=column,State1,$good -print=column,State2,$good -noWarning} result] {
        return -code error "APSSCRPrintXrayBPMCompTable: $result"
    } 
    set rows1 [exec sdds2stream ${tmpRoot}.print1 -rows=bare]
    if [catch {exec sddsprocess $resultsFile  ${tmpRoot}.print2 \
                 -filter=column,minmaxV1,0,$minVoltage \
                 -filter=column,minmaxV2,0,$minVoltage,! \
                 "-print=parameter,Title,Comparison of\n$snap1Label\n to \n$snap2Label\n \
               (minimal acceptable voltage $minVoltage V \n minimal acceptable position difference $posDiffTolerance mm)" \
                 -print=column,State1,$questionable -print=column,State2,$good -noWarning} result] {
        return -code error "APSSCRPrintXrayBPMCompTable: $result"
    }   
    set rows2 [exec sdds2stream ${tmpRoot}.print2 -rows=bare]
    
    if [catch {exec sddsprocess $resultsFile  ${tmpRoot}.print3 \
                 -filter=column,minmaxV1,0,$minVoltage,! \
                 -filter=column,minmaxV2,0,$minVoltage \
                 "-print=parameter,Title,Comparison of\n$snap1Label\n to \n$snap2Label\n \
               (minimal acceptable voltage $minVoltage V)" \
                 -print=column,State1,$good -print=column,State2,$questionable  -noWarning} result] {
        return -code error "APSSCRPrintXrayBPMCompTable: $result" 
    }
    set rows3 [exec sdds2stream ${tmpRoot}.print3 -rows=bare]
  
    if [catch {exec sddsprocess $resultsFile  ${tmpRoot}.print4 \
                 -filter=column,minmaxV1,0,$minVoltage \
                 -filter=column,minmaxV2,0,$minVoltage \
                 "-print=parameter,Title,Comparison of\n$snap1Label\n to \n$snap2Label\n \
               (minimal acceptable voltage $minVoltage V)" \
                 -print=column,State1,$questionable -print=column,State2,$questionable  -noWarning} result] {
        return -code error "APSSCRPrintXrayBPMCompTable: $result"  
    }  
    set rows4 [exec sdds2stream ${tmpRoot}.print4 -rows=bare]
    
    
    if {$rows1 > 0} {lappend mergedFiles ${tmpRoot}.print1}        
    if {$rows2 > 0} {lappend mergedFiles ${tmpRoot}.print2}
    if {$rows3 > 0} {lappend mergedFiles ${tmpRoot}.print3}
    if {$rows4 > 0} {lappend mergedFiles ${tmpRoot}.print4}
    
    APSAddToTmpFileList -ID compareXrayBPM -fileList $mergedFiles
    APSAddToTmpFileList -ID compareXrayBPM -fileList ${tmpRoot}.1
    catch {eval exec sddscombine $mergedFiles ${tmpRoot}.1 -merge -overWrite}
    if {$bpmType=="ID"} {
       # set gapThreshold 185
        foreach bpm {1 2} {
            if [catch {exec sddsconvert ${tmpRoot}.1 $tmpRoot.${plane}$bpm \
                         -retain=col,Rootname,gap,USgap,${plane}bpm${bpm}*,State${bpm}
                exec sddsprocess $tmpRoot.${plane}$bpm $tmpRoot.bad${plane}$bpm -noWarnings \
                           "-filter=col,${plane}bpm${bpm}Diff,-$posDiffTolerance,$posDiffTolerance,!" \
                         "-reprint=col,State$bpm,%s bad,State${bpm}" 
                exec sddsselect $tmpRoot.${plane}$bpm $tmpRoot.bad${plane}$bpm -match=Rootname \
                           -invert -nowarnings -pipe=out\
                         | sddsconvert -pipe=in $tmpRoot.good${plane}$bpm -rename=col,State=State$bpm
                exec sddscombine $tmpRoot.good${plane}$bpm $tmpRoot.bad${plane}$bpm -merge -pipe=out \
                         | sddssort -pipe=in -col=Rootname ${tmpRoot}.bpm$bpm } result] {
                return -code error "PrintCompareTable: $result"
            }
        }
        if [catch {exec sddsxref ${tmpRoot}.bpm1 ${tmpRoot}.bpm2 -match=Rootname \
                     -take=${plane}bpm2*,State2 ${tmpRoot}.${plane} } result] {
            return -code error "PrintCompareTable: $result"
        }
    } else {
        if {$plane!="y"} {
            return -code error "PrintCompareTable: invalid plane given for compare BM BPMs (has to be y)" 
        }
        foreach bpm {1 2} {
            if [catch {exec sddsconvert ${tmpRoot}.1 $tmpRoot.y$bpm \
                         -retain=col,Rootname,ybpm${bpm}*,State${bpm}
                exec sddsprocess $tmpRoot.y$bpm $tmpRoot.bady$bpm -noWarnings \
                         "-filter=col,ybpm${bpm}Diff,-$posDiffTolerance,$posDiffTolerance,!" \
                         "-reprint=col,State$bpm,%s bad,State${bpm}" 
                exec sddsselect $tmpRoot.y$bpm $tmpRoot.bady$bpm -match=Rootname \
                         -invert -nowarnings -pipe=out  \
                         | sddsconvert -pipe=in $tmpRoot.goody$bpm -rename=col,State=State$bpm
                exec sddscombine $tmpRoot.goody$bpm $tmpRoot.bady$bpm -merge -pipe=out \
                         | sddsprocess -pipe -noWarnings \
                         | sddssort -pipe=in -col=Rootname ${tmpRoot}.bpm$bpm } result] {
                return -code error "PrintCompareTable: $result"
            }
        }
        if [catch {exec sddsxref ${tmpRoot}.bpm1 ${tmpRoot}.bpm2 -match=Rootname \
                     -take=ybpm2*,State2 ${tmpRoot}.y } result] {
            return -code error "PrintCompareTable: $result"
        }
    }
    APSAddToTmpFileList -ID compareXrayBPM -fileList "${tmpRoot}.${plane}1 ${tmpRoot}.${plane}2 ${tmpRoot}.bad${plane}1 ${tmpRoot}.bad${plane}2 ${tmpRoot}.good${plane}1 ${tmpRoot}.good${plane}2 ${tmpRoot}.$plane ${tmpRoot}.bpm1 ${tmpRoot}.bpm2"
    set title [exec sdds2stream ${tmpRoot}.$plane -par=Title]
    switch $bpmType {
        ID {
            set dlina 105
            set title "$title\n\n                           --------------P1 ${plane}--------------------     -------------P2 ${plane}---------------------\n"
            exec sddsprintout ${tmpRoot}.${plane} ${tmpRoot}.printout "-title=$title" \
              -col=Rootname,label=ID,format=%6s \
              -col=gap,format=%6.2f,label=ds:gap \
              -col=USgap,format=%6.2f,label=us:gap \
              -col=${plane}bpm1,label=$Value1Label,format=%7.3f \
              -col=${plane}bpm1New,label=$Value2Label,format=%7.3f \
              "-col=${plane}bpm1Diff,label=$DifferenceLabel,format=%7.3f" \
              -col=State1,label=State,format=%8s \
              -col=${plane}bpm2,label=$Value1Label,format=%7.3f \
              -col=${plane}bpm2New,label=$Value2Label,format=%7.3f \
              "-col=${plane}bpm2Diff,label=$DifferenceLabel,format=%7.3f" \
              -col=State2,label=State,format=%8s
        }
        BM {    
            set dlina 75
            set title "$title\n\n         -----------------P1 ${plane}-----------------   -----------P2 ${plane}-----------------------\n"
            exec sddsprintout ${tmpRoot}.y ${tmpRoot}.printout "-title=$title" \
              -col=Rootname,label=BM,format=%6s \
              -col=ybpm1,label=$Value1Label,format=%7.3f \
              -col=ybpm1New,label=$Value2Label,format=%7.3f \
              "-col=ybpm1Diff,label=$DifferenceLabel,format=%7.3f" \
              -col=State1,label=State,format=%8s \
              -col=ybpm2,label=$Value1Label,format=%7.3f \
              -col=ybpm2New,label=$Value2Label,format=%7.3f \
              "-col=ybpm2Diff,label=$DifferenceLabel,format=%7.3f" \
              -col=State2,label=State,format=%8s
        }
    }
    APSFileDisplayWindow [APSUniqueName .] -fileName ${tmpRoot}.printout \
      -deleteOnClose 1 -width 110 -height 45 -printCommand "enscript -r"
    return $tmpRoot
}

proc APSSRPlotSCRBPMData {args} {
    set field OffsetAO
    set machine ""
    set snapshot ""
    APSStrictParseArguments {field machine snapshot}
    global SCRdebug apsSCRTopDirectory
    
    if {![string length $field] || ![string length $machine] || ![string length $snapshot]} {
        return -code error "APSSRPlotControlNameData: bad arguments"
    }

    set title "$field values from [file tail [file rootname $snapshot]]"
    set tmpRoot /tmp/[APSTmpString]
    set tmpID [APSUniqueName tmpID]
    APSAddToTmpFileList -ID $tmpID \
      -fileList [list $tmpRoot.x $tmpRoot.y $tmpRoot.print $tmpRoot.xy]

    foreach plane {x y} {
        if [catch {exec sddsprocess $snapshot -pipe=out -nowarning \
                     -match=column,ControlName=*:ms:$plane:$field \
                     -edit=column,BPMName,ControlName,%/:ms:$plane:$field// \
                     -scan=column,$plane:$field,ValueString,%le,type=double,units=mm \
                     | sddssort -pipe=in $tmpRoot.$plane -numericHigh -column=BPMName } result] {
            return -code error "APSSRPlotSCRBPMData: $result"
        }
    }
    if [catch {exec sddsxref $tmpRoot.x $tmpRoot.y -pipe=out -nowarning \
                 -take=* -match=BPMName -fillIn \
                 | tee $tmpRoot.xy  \
                 | sddsprintout -pipe=in $tmpRoot.print \
                 -column=BPMName -column=x:$field -column=y:$field} result] {
        return -code error "APSSRPlotSCRBPMData: $result"
    }

    exec sddsplot -layout=1,2 $tmpRoot.xy -graph=impulse -sep \
      -column=BPMName,x:$field -column=BPMName,y:$field & 
    APSFileDisplayWindow [APSUniqueName .] \
      -comment "$title" -fileName $tmpRoot.print -deleteOnClose 1 -width 80 \
      -contextHelp "$title" -sddsExportableFile $tmpRoot.xy
}


