#
# $Log: not supported by cvs2svn $
# Revision 1.20  2003/09/24 15:12:28  soliday
# It does not stop anymore if logDaemon is not running.
#
# Revision 1.19  2003/03/26 08:42:49  emery
# Added BTS controllaw suspend.
#
# Revision 1.18  2002/06/11 20:30:04  soliday
# Added some extra log messages.
#
# Revision 1.17  2001/03/04 01:35:53  emery
# replaced exec rm with file delete
#
# Revision 1.16  2000/04/26 15:46:30  soliday
# Added APSMpStep descriptions.
#
# Revision 1.15  1998/05/25 00:30:08  borland
# Now allows retaking the data an arbitrary number of times, with plots
# for each attempt.
#
# Revision 1.14  1998/01/19 20:48:56  borland
# Made some improvements to allow excluding BPMs and to properly handle case
# when actuators are not used.  Old version would pick up files from previous
# measurement or bomb when it didn't find all the files.
#
# Revision 1.13  1997/08/14 05:59:46  emery
# Replacing an if with a while loop for the
# APSMpRespMatrixDoExperiment command so that an actuator
# can be repeatedly measured until the data is ok, or the user
# gives up.
#
# Revision 1.12  1997/08/14 04:35:47  borland
# Added ability to measure ES1, ES2, or no septum.  Also option to
# measure no correctors.
#
# Revision 1.11  1997/03/05 03:46:34  emery
# Use variable fileList to save one execution of the APSAffix command.
#
# Revision 1.10  1997/01/05 10:06:03  emery
# Added the catch to the other occurence of sddscombine
#
# Revision 1.9  1997/01/05 10:03:10  emery
# Put sddscombine of *.def files in a catch so that
# if any of the files in the input list don't exist,
# then the procedure will forge ahead.
#
# Revision 1.8  1997/01/05 09:43:14  emery
# Corrected the logic of the if argument
# which used the value of promptExperiments.
#
# Revision 1.7  1997/01/05 09:07:23  emery
# Changed actuator B:ES1 to B:ES2 which is the
# booster septum that drifts.
#
# Revision 1.6  1996/12/20 21:05:52  emery
# added capability of skipping correctors.
#
# Revision 1.5  1996/12/08 10:26:22  emery
# Surrounded a exec rm statement with a catch
#
# Revision 1.4  1996/10/08 17:28:54  borland
# Added proper CVS header.
#
#

proc APSMpxBTSMatrixInfo {} {
    return "This procedure measures the BTS response matrix in the horizontal plane, calculates the inverse matrix, and gives the user a choice to install this inverse matrix as the matrix for the xBTS controllaw."
}

proc APSMpxBTSMatrix {args} {
    global status

    APSParseArguments {}

# the final result (.def and inverse matrix file) go in the resultsDir
# the temporary results go the dataDir.
    set resultsDir /home/helios/oagData/controllaw/BTS/BTSonly
    set dataDir $resultsDir/measurement
    set installName xBTS.matrix
    set beamline BTS
    set beamlineFile $dataDir/${beamline}.beamline
    set plane H
    set matrix R12
    set templateFile $dataDir/${beamline}:${plane}Template.exp
    set actuatorList {}
    # default for filterColumns in procedure formMatrix is "*:*P*"
    set filterColumns "$beamline:\[ABC\]P${plane}*"
    set excludeColumns ""

    cd $dataDir

    # Setting up experiment parameters
    set halfRange 4.0
    set points 5
    set postChange 2.5
    set intermeasurement 0.5
    set average 10
    set checkResults 0
    set promptExperiments 0
    set useSeptum Both
    set useCorrectors 1

    APSMpStep init -promptDialog APSMpxBTSMatrixInitDialog -vars {halfRange points postChange intermeasurement average checkResults filterColumns promptExperiments useSeptum useCorrectors excludeColumns}
        
    foreach param "halfRange points postChange intermeasurement average checkResults filterColumns promptExperiments useSeptum useCorrectors excludeColumns" {
        append paramString "$param=\"[set $param]\" "
    }
    if {[catch {APSLogScriptAction \
		    -procedure APSMpxBTSMatrix \
		    -action initialize \
		    -state ok \
		    -parameters $paramString} results]} {
#	APSMpReturn error "APSMpxBTSMatrix: $results"
    }

    switch $useSeptum {
        Both {set actuatorList "B:ES1 B:ES2"}
        ES1 {set actuatorList B:ES1}
        ES2 {set actuatorList B:ES2}
        None {}
        default {set actuatorList B:ES2}
    }
    if $useCorrectors {
        set corrList {AH1 AH2 AH3 BH1 BH2 CH1}
        eval lappend actuatorList [APSAffix $corrList -prefix ${beamline}:]
    }
    if ![llength $actuatorList] {
        APSMpReturn error "No actuators chosen"
    }

# this procedure establishes some default variables and arrays for parameters.
    set parameterList {PV halfRange points postChange intermeasurement average}
    APSMpRespMatrixSetParameterDefaults \
      -parameterList $parameterList \
      -halfRange $halfRange \
      -points $points \
      -postChange $postChange \
      -intermeasurement $intermeasurement \
      -average $average
    foreach actuator $actuatorList {
        if {[lsearch -exact [list B:ES2 B:ES1] $actuator]!=-1} {
# this procedure must be preceded by APSMpRespMatrixSetParameterDefaults
# in order for the arguments to be recognized.
            APSMpRespMatrixSetActuatorParameters \
              -actuator $actuator \
              -PV ${actuator}:VoltageSetSendAO \
              -halfRange 5
        } else {
            APSMpRespMatrixSetActuatorParameters \
              -actuator $actuator \
              -PV ${actuator}:CurrentAO 
        }
    }
    #  bad equipment to remove from lists
    set badActuators ""
    set badBpms ""

    # put pre-execution commands here
    # such as reseting the time-outs for the pulsed magnets.
    APSMpStep PreExecutionCommands -desc "Do pre-execution commands, such as reseting the time-outs for the pulsed magnets, or suspending controllaws."
    set rcList {BTS:ControlLawXRC BTS:ControlLawYRC}
    if [catch {exec cavget -list=[join $rcList ,] \
                 -list=.SUSP -cavputForm} \
          controllawMode] {
        APSMpReturn error "APSMpxBTSMatrix: Problem getting BTS controllaw status: $controllawMode"
        return
    }
    if [catch {exec cavput -list=[join $rcList ,] \
                 -list=.SUSP=1} result] {
        APSMpReturn error "APSMpxBTSMatrix: Problem suspending BTS controllaws: $result"
    }

    if [catch {APSMpBES2Feedforward -action stop; after 1000; \
                 APSMpBES2Feedforward -action start \
             } result] {
        APSMpReturn error "APSMpxBTSMatrix: Problem stopping and restarting BES2 feedforwar: $result"
    }

    # take data
    APSMpStep SDDSExperimentCommands -desc "Take data"
    # send a list of one actuator at a time so that the data can be reviewed
    # and restarted at every step
    set actuatorList1 ""
    foreach actuator $actuatorList {
        if [file exists $actuator.sdds] {
            exec mv $actuator.sdds $actuator.sdds~
        }
        if {!$promptExperiments || [APSYesNoPopUp "Do experiment on $actuator"]} {
            APSMpRespMatrixDoExperiment -actuatorList $actuator -templateFile $templateFile
            catch {eval file delete [glob *~]}
            if $checkResults {
                eval exec sddsplot -col=$actuator,(*) $actuator.sdds \
                  -grap=sym,conn,sca=3 -sep &
                while {[APSYesNoPopUp "Retake data? (wait for plot to appear)"]} {
                    APSMpRespMatrixSetActuatorParametersByDialog .dialog -actuator $actuator
                    APSMpRespMatrixDoExperiment -actuatorList $actuator \
                      -templateFile $templateFile
                    eval exec sddsplot -col=$actuator,(*) $actuator.sdds \
                      -grap=sym,conn,sca=3 -sep &
                }
            }
            lappend actuatorList1 $actuator
        }
    }
    set actuatorList $actuatorList1

    # put post-execution commands here
    APSMpStep PostExecutionCommands -desc "Do post-execution commands"
    if {[catch {exec cavput -list=$controllawMode} result]} {
        APSMpReturn error "APSMpxBTSMatrix: Problem setting BTS controllaw mode: $result"
    }

    APSMpStep CombiningResults -desc "Combine results"
    set fileList [eval glob [APSAffix $actuatorList -suffix .def]]
    catch {eval exec sddscombine $fileList -pipe=output -merge -overWrite \
             | sddssort -pipe=input ${beamline}.${plane}.def -col=ControlName  -unique}
    catch {eval file delete $fileList}
    
    APSMpStep CalculatingSlopes -desc "Calculate slopes" -proc "APSMpRespMatrixCalcSlopes"
    APSMpRespMatrixCalcSlopes -actuatorList $actuatorList 

    APSMpStep FormingMatrix -desc "Form matrix" -proc "APSMpRespMatrixFormMatrix"
    APSMpRespMatrixFormMatrix -actuatorList $actuatorList -badBpms $badBpms -badActuators $badActuators \
      -outFile ${beamline}.${matrix} -beamlineFile $beamlineFile -filterColumns $filterColumns \
      -excludeColumns $excludeColumns

    APSMpStep CalculatingInverse -desc "Calculate inverse" -proc "APSMpRespMatrixCalcInverse"
    APSMpRespMatrixCalcInverse -inFile ${beamline}.${matrix} -outFile ${beamline}.Inv${matrix} -minimum 0.1

    APSMpStep InstallMatrix -desc "Install matrix" -proc "APSMpRespMatrixInstallDialog"

    puts "APSMpxBTSMatrix:fileName ${beamline}.Inv${matrix}"
    puts "APSMpxBTSMatrix:installName $installName"

    APSMpRespMatrixInstallDialog .install -fileName ${beamline}.Inv${matrix} \
      -installName $installName -installDir $resultsDir

    APSMpReturn ok
}


# Note: by convention, this procedure has only one argument. It provides the
#    parent frame into which you should pack all widgets.
proc    APSMpxBTSMatrixInitDialog {frame} {
  # Note: this dialog procedure has access only to those variables explicitly
  #    passed in via the -vars option of APSMpStep. You can access them here
  #    as elements of the following global array (which always has the same
  #    name as the machine procedure itself.
  global APSMpxBTSMatrix
  label $frame.label -text "Enter default values for sddsexperiment"
  pack $frame.label
  APSLabeledEntry .le1 -parent $frame -label "half range" -textVariable \
    APSMpxBTSMatrix(halfRange)
  APSLabeledEntry .le2 -parent $frame -label "no. points" -textVariable \
    APSMpxBTSMatrix(points)
  APSLabeledEntry .le3 -parent $frame -label "post change pause" -textVariable \
    APSMpxBTSMatrix(postChange)
  APSLabeledEntry .le4a -parent $frame -label "filter for bpm names" -textVariable \
    APSMpxBTSMatrix(filterColumns)
  APSLabeledEntry .le4c -parent $frame -label "bpm names to exclude" -textVariable \
    APSMpxBTSMatrix(excludeColumns)
  APSLabeledEntry .le4b -parent $frame -label "intermeasurement pause" -textVariable \
    APSMpxBTSMatrix(intermeasurement)
  APSLabeledEntry .le5 -parent $frame -label "no. average" -textVariable \
    APSMpxBTSMatrix(average)
  APSRadioButtonFrame .rb6 -parent $frame -label "Prompt for each experiment?" \
    -variable APSMpxBTSMatrix(promptExperiments) -buttonList {Yes No} -valueList {1 0} \
    -orientation horizontal 
  APSRadioButtonFrame .rb7 -parent $frame -label "Check results?" -variable \
    APSMpxBTSMatrix(checkResults) -buttonList {Yes No} -valueList {1 0} \
    -orientation horizontal 
  APSRadioButtonFrame .rb8 -parent $frame -label "Use correctors: " -variable \
    APSMpxBTSMatrix(useCorrectors) -buttonList {Yes No} -valueList {1 0} \
    -orientation horizontal
  APSRadioButtonFrame .rb1 -parent $frame -label "Use septum: " -variable \
    APSMpxBTSMatrix(useSeptum) -buttonList {ES1 ES2 Both None} \
    -valueList {ES1 ES2 Both None} -orientation horizontal 
}

proc APSMpyBTSMatrixInfo {} {
    return "This procedure measures the BTS response matrix in the vertical plane, calculates the inverse matrix, and gives the user a choice to install this inverse matrix as the matrix for the yBTS controllaw."
}

proc APSMpyBTSMatrix {args} {
    global status

    APSParseArguments {}

# the final result (.def and inverse matrix file) go in the resultsDir
# the temporary results go the dataDir.
    set resultsDir /home/helios/oagData/controllaw/BTS/BTSonly
    set dataDir $resultsDir/measurement
    set installName yBTS.matrix
    set beamline BTS
    set beamlineFile $dataDir/${beamline}.beamline
    set plane V
    set matrix R34
    set templateFile $dataDir/${beamline}:${plane}Template.exp
    set actuatorList {AV1 AV2 AV3 BV1 CV1}
    set actuatorList [APSAffix $actuatorList -prefix ${beamline}:]
    # default for filterColumns in procedure formMatrix is "*:*P*"
    set filterColumns "$beamline:\[ABC\]P${plane}*"
    set excludeColumns "$beamline:BPV2:y"

    cd $dataDir

    # Setting up experiment parameters
    set halfRange 4.0
    set points 5
    set postChange 2.5
    set intermeasurement 0.5
    set average 10
    set checkResults 0
    set promptExperiments 0
    APSMpStep init -promptDialog APSMpyBTSMatrixInitDialog -vars {halfRange points postChange intermeasurement average checkResults filterColumns promptExperiments}

    foreach param "halfRange points postChange intermeasurement average checkResults filterColumns promptExperiments" {
        append paramString "$param=\"[set $param]\" "
    }
    if {[catch {APSLogScriptAction \
		    -procedure APSMpyBTSMatrix \
		    -action initialize \
		    -state ok \
		    -parameters $paramString} results]} {
#	APSMpReturn error "APSMpyBTSMatrix: $results"
    }

# this procedure establishes some default variables and arrays for parameters.
    set parameterList {PV halfRange points postChange intermeasurement average}
    APSMpRespMatrixSetParameterDefaults \
      -parameterList $parameterList \
      -halfRange $halfRange \
      -points $points \
      -postChange $postChange \
      -intermeasurement $intermeasurement \
      -average $average
    foreach actuator $actuatorList {
# this procedure must be preceded by APSMpRespMatrixSetParameterDefaults
# in order for the arguments to be recognized.
        APSMpRespMatrixSetActuatorParameters \
          -actuator $actuator \
          -PV ${actuator}:CurrentAO 
    }
    #  bad equipment to remove from lists
    set badActuators ""
    set badBpms ""

    # put pre-execution commands here
    # such as reseting the time-outs for the pulsed magnets.
    APSMpStep PreExecutionCommands -desc "Do pre-execution commands, such as reseting the time-outs for the pulsed magnets"
    set rcList {BTS:ControlLawXRC BTS:ControlLawYRC}
    if [catch {exec cavget -list=[join $rcList ,] \
                 -list=.SUSP -cavputForm} \
          controllawMode] {
        APSMpReturn error "APSMpxBTSMatrix: Problem getting BTS controllaw status: $controllawMode"
        return
    }
    if [catch {exec cavput -list=[join $rcList ,] \
                 -list=.SUSP=1} result] {
        APSMpReturn error "APSMpxBTSMatrix: Problem suspending BTS controllaws: $result"
    }

    # take data
    APSMpStep SDDSExperimentCommands -desc "Take data"
    # send a list of one actuator at a time so that the data can be reviewed
    # and restarted at every step
    set actuatorList1 ""
    foreach actuator $actuatorList {
        if [file exists $actuator.sdds] {
            exec mv $actuator.sdds $actuator.sdds~
        }
        if {!$promptExperiments || [APSYesNoPopUp "Do experiment on $actuator"]} {
            APSMpRespMatrixDoExperiment -actuatorList $actuator -templateFile $templateFile
            catch {eval file delete [glob *~]}
            if $checkResults {
                eval exec sddsplot -col=$actuator,(*) $actuator.sdds \
                  -grap=sym,conn,sca=3 -sep &
                while {[APSYesNoPopUp "Retake data? (wait for plot to appear)"]} {
                    APSMpRespMatrixSetActuatorParametersByDialog .dialog -actuator $actuator
                    APSMpRespMatrixDoExperiment -actuatorList $actuator \
                      -templateFile $templateFile
                    eval exec sddsplot -col=$actuator,(*) $actuator.sdds \
                      -grap=sym,conn,sca=3 -sep &
                }
            }
            lappend actuatorList1 $actuator
        }
    }
    set actuatorList $actuatorList1
    
    # put post-execution commands here
    APSMpStep PostExecutionCommands -desc "Do post-execution commands"
    if {[catch {exec cavput -list=$controllawMode} result]} {
        APSMpReturn error "APSMpxBTSMatrix: Problem setting BTS controllaw mode: $result"
    }

    APSMpStep CombiningResults -desc "Combine results"
    set fileList [eval glob [APSAffix $actuatorList -suffix .def]]
    catch {eval exec sddscombine $fileList -pipe=output -merge -overWrite \
             | sddssort -pipe=input ${beamline}.${plane}.def -col=ControlName  -unique }
    catch {eval file delete $fileList}

    APSMpStep CalculatingSlopes -desc "Calculate slopes" -proc "APSMpRespMatrixCalcSlopes"
    APSMpRespMatrixCalcSlopes -actuatorList $actuatorList 

    APSMpStep FormingMatrix -desc "Form matrix" -proc "APSMpRespMatrixFormMatrix"
    APSMpRespMatrixFormMatrix -actuatorList $actuatorList -badBpms $badBpms -badActuators $badActuators \
      -outFile ${beamline}.${matrix} -beamlineFile $beamlineFile -filterColumns $filterColumns \
      -excludeColumns $excludeColumns

    APSMpStep CalculatingInverse -desc "Calculate inverse" -proc "APSMpRespMatrixCalcInverse"
    APSMpRespMatrixCalcInverse -inFile ${beamline}.${matrix} -outFile ${beamline}.Inv${matrix} -minimum 0.1

    APSMpStep InstallMatrix -desc "Install matrix" -proc "APSMpRespMatrixInstallDialog"
    APSMpRespMatrixInstallDialog .install -fileName ${beamline}.Inv${matrix} \
      -installName $installName -installDir $resultsDir
    APSMpReturn ok
}


# Note: by convention, this procedure has only one argument. It provides the
#    parent frame into which you should pack all widgets.
proc    APSMpyBTSMatrixInitDialog {frame} {
  # Note: this dialog procedure has access only to those variables explicitly
  #    passed in via the -vars option of APSMpStep. You can access them here
  #    as elements of the following global array (which always has the same
  #    name as the machine procedure itself.
  global APSMpyBTSMatrix
  label $frame.label -text "Enter default values for sddsexperiment"
  pack $frame.label
  APSLabeledEntry .le1 -parent $frame -label "half range" -textVariable \
    APSMpyBTSMatrix(halfRange)
  APSLabeledEntry .le2 -parent $frame -label "no. points" -textVariable \
    APSMpyBTSMatrix(points)
  APSLabeledEntry .le3 -parent $frame -label "post change pause" -textVariable \
    APSMpyBTSMatrix(postChange)
  APSLabeledEntry .le4a -parent $frame -label "filter for bpm names" -textVariable \
    APSMpyBTSMatrix(filterColumns)
  APSLabeledEntry .le4c -parent $frame -label "bpm names to exclude" -textVariable \
    APSMpyBTSMatrix(excludeColumns)
  APSLabeledEntry .le4b -parent $frame -label "intermeasurement pause" -textVariable \
    APSMpyBTSMatrix(intermeasurement)
  APSLabeledEntry .le5 -parent $frame -label "no. average" -textVariable \
    APSMpyBTSMatrix(average)
  APSLabeledEntry .le6 -parent $frame -label "prompt for each experiment?" -textVariable \
    APSMpyBTSMatrix(promptExperiments)
  APSLabeledEntry .le7 -parent $frame -label "check results?" -textVariable \
    APSMpyBTSMatrix(checkResults)
}
